@@ -887,6 +887,28 @@ static void ParseFile(const char *filename, int is_stdin)
 		{
 			CCStringListAdd(&argv, &argc, "-undef");
 		}
+		if(preproc_show_headers)
+		{
+			CCStringListAdd(&argv, &argc, "-H");
+		}
+		if(dependency_level != 0)
+		{
+			/* Generate dependencies while pre-processing */
+			static char * const depLevels[] = {0, "-M", "-MD", "-MM", "-MMD"};
+			CCStringListAdd(&argv, &argc, depLevels[dependency_level]);
+			if(dependency_level == DEP_LEVEL_MD ||
+		       dependency_level == DEP_LEVEL_MMD)
+			{
+				if(dependency_file != 0)
+				{
+					CCStringListAdd(&argv, &argc, dependency_file);
+				}
+			}
+			if(dependency_gen_flag)
+			{
+				CCStringListAdd(&argv, &argc, "-MG");
+			}
+		}
 		if(dump_output_format == DUMP_MACROS_ONLY)
 		{
 			CCStringListAdd(&argv, &argc, "-dM");
