@@ -935,6 +935,8 @@ static void CreateMethod(ILGenInfo *info, ILClass *classInfo,
 {
 	char *name;
 	char *basicName;
+	ILUInt32 thisAccess;
+	ILUInt32 baseAccess;
 	ILType *tempType;
 	ILMethod *methodInfo;
 	ILType *signature;
@@ -967,7 +969,7 @@ static void CreateMethod(ILGenInfo *info, ILClass *classInfo,
 		if(signature)
 		{
 			if(!ILType_IsClass(signature) ||
-			   !ILClass_IsInterface(ILType_ToClass(signature)))
+			   !ILClass_IsInterface(ILClassResolve(ILType_ToClass(signature))))
 			{
 				CCErrorOnLine(yygetfilename(method), yygetlinenum(method),
 							  "`%s' is not an interface",
@@ -975,7 +977,7 @@ static void CreateMethod(ILGenInfo *info, ILClass *classInfo,
 			}
 			else
 			{
-				interface = ILType_ToClass(signature);
+				interface = ILClassResolve(ILType_ToClass(signature));
 
 				/* Modify the method name to include the fully-qualified
 				   form of the interface's class name */
@@ -1222,13 +1224,23 @@ static void CreateMethod(ILGenInfo *info, ILClass *classInfo,
 			  			"declaration of `%s' overrides an inherited member, "
 						"and `override' was not present", name);
 				}
-				if((method->modifiers & CS_SPECIALATTR_OVERRIDE) !=0 
-				 && (method->modifiers & IL_META_METHODDEF_MEMBER_ACCESS_MASK)
-				 !=	(ILMember_Attrs(member) & 
-						 IL_META_METHODDEF_MEMBER_ACCESS_MASK))
+
+				/* Get the access modifiers for this and the base methods */
+				thisAccess = (method->modifiers &
+				            IL_META_METHODDEF_MEMBER_ACCESS_MASK);
+				baseAccess = (ILMember_Attrs(member) &
+				            IL_META_METHODDEF_MEMBER_ACCESS_MASK);
+
+				/* Check for legal modifiers for overrides */
+				if((method->modifiers & CS_SPECIALATTR_OVERRIDE) != 0 &&
+				   (thisAccess != baseAccess) &&
+				   ((ILProgramItem_Image(member) ==
+				     ILProgramItem_Image(methodInfo)) ||
+				    (thisAccess != IL_META_METHODDEF_FAMILY) ||
+				    (baseAccess != IL_META_METHODDEF_FAM_OR_ASSEM)))
 				{
-					class1=ILMember_Owner(member);
-					class2=ILMethod_Owner(methodInfo);
+					class1 = ILMember_Owner(member);
+					class2 = ILMethod_Owner(methodInfo);
 					CCErrorOnLine(yygetfilename(method), yygetlinenum(method),
 						"cannot change the access modifiers while overriding "
 						"method '%s%s%s.%s' with '%s%s%s.%s' ",
