@@ -117,6 +117,11 @@ extern	"C" {
 	#define	IL_MEMSET(dst,ch,len)			(ILMemSet((dst), (ch), (len)))
 	#define	IL_MEMCMP(dst,src,len)			(ILMemCmp((dst), (src), (len)))
 #endif
+	
+#if !defined(IL_CONFIG_REDUCE_CODE) && !defined(IL_WITHOUT_TOOLS)
+/* Global lock for trace outputs */
+ILMutex *globalTraceMutex;
+#endif
 
 /*
  * Modify the program counter and stack pointer.
@@ -613,6 +618,7 @@ int _ILCVMInterpreter(ILExecThread *thread)
 				/* We don't need "prefix" in direct mode, so just stub it out */
 				switch(1)
 				{
+				case 0:
 					VMPREFIXDEFAULT:
 					{
 						MODIFY_PC_AND_STACK(CVM_LEN_NONE, 0);
