@@ -523,6 +523,20 @@ extern	"C" {
  */
 #define	COP_PREFIX_WADDR_NATIVE_N	0x5F
 
+/*
+ * Method Trace Instructions
+ */
+#define COP_PREFIX_TRACE_IN		0x60
+#define COP_PREFIX_TRACE_OUT		0x61
+
+/*
+ * More prefixed exception handling opcodes.
+ */
+#define COP_PREFIX_START_CATCH			0x62
+#define COP_PREFIX_START_FINALLY		0x63
+#define	COP_PREFIX_PROPAGATE_ABORT		0x64
+
+
 /*
  * Definition of a CVM stack word which can hold
  * either 32-bit quantities or pointers.
