@@ -18,6 +18,9 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include "il_dumpasm.h"
+#include "interlocked.h"
+
 #if defined(IL_CVM_GLOBALS)
 
 /*
@@ -30,6 +33,8 @@
 #define	CVM_OPTIMIZE_BLOCK()
 #endif
 
+//#define INDENT_TRACE
+
 /*
  * Call a function via the FFI interface.  If we have "libffi",
  * then use the "ffi_call" and "ffi_raw_call" functions.
@@ -153,11 +158,44 @@ ILCallFrame *_ILAllocCallFrame(ILExecThread *thread)
 				} \
 			} while (0)
 
+#define CHECK_ABORT()	\
+	if (thread->abortRequested && ILThreadIsAbortRequested())	\
+	{	\
+		thread->abortRequested = 0;	\
+		/* Mark the thread as aborted if it hasn't already been aborted	*/ \
+		if (ILThreadSelfAborting(thread->supportThread))	\
+		{	\
+			/* Great, thread hasn't aborted itself yet so throw a ThreadAbortException. */ \
+			stacktop[0].ptrValue = _ILExecThreadNewThreadAbortException(thread, 0); \
+			thread->aborting = 1; \
+			thread->abortHandlerEndPC = 0; \
+			thread->abortHandlerFrame = 0; \
+			thread->threadAbortException = 0; \
+			stacktop += 1; \
+			thread->runningManagedCode = 1;	\
+			if (!ILCoderPCToHandler(thread->process->coder, pc, 0)) \
+			{ \
+				goto throwException; \
+			} \
+			else \
+			{ \
+				goto prefixThrowException; \
+			} \
+		}	\
+	}
+
+#define BEGIN_NATIVE_CALL()	\
+	thread->runningManagedCode = 0;
+
+#define END_NATIVE_CALL() \
+	CHECK_ABORT(); \
+	thread->runningManagedCode = 1;
+
 /*
  * Determine the number of stack words that are occupied
  * by a specific type.
  */
-static ILUInt32 StackWordsForType(ILExecThread *thread, ILType *type)
+ILUInt32 _ILStackWordsForType(ILExecThread *thread, ILType *type)
 {
 	if(type == ILType_Float32 || type == ILType_Float64)
 	{
@@ -177,7 +215,7 @@ static ILUInt32 StackWordsForType(ILExecThread *thread, ILType *type)
  * Returns the number of stack words to pop from the function,
  * and the new array in "*array".
  */
-static ILUInt32 PackVarArgs(ILExecThread *thread, CVMWord *stacktop,
+ILUInt32 _ILPackVarArgs(ILExecThread *thread, CVMWord *stacktop,
 							ILUInt32 firstParam, ILUInt32 numArgs,
 							ILType *callSiteSig, void **array)
 {
@@ -208,7 +246,7 @@ static ILUInt32 PackVarArgs(ILExecThread *thread, CVMWord *stacktop,
 	height = 0;
 	for(param = 0; param < numArgs; ++param)
 	{
-		height += StackWordsForType
+		height += _ILStackWordsForType
 			(thread, ILTypeGetParam(callSiteSig, firstParam + param));
 	}
 	stacktop -= height;
@@ -218,121 +256,249 @@ static ILUInt32 PackVarArgs(ILExecThread *thread, CVMWord *stacktop,
 	for(param = 0; param < numArgs; ++param)
 	{
 		paramType = ILTypeGetParam(callSiteSig, firstParam + param);
-		if(ILType_IsPrimitive(paramType))
+
+		if (ILType_IsComplex(paramType)
+			&& ILType_Kind(paramType) == IL_TYPE_COMPLEX_BYREF)
 		{
-			/* Box a primitive value after aligning it properly */
-			switch(ILType_ToElement(paramType))
+			ILType *targetParamType = ILType_Ref(paramType);
+
+			if(ILType_IsPrimitive(targetParamType))
 			{
-				case IL_META_ELEMTYPE_I1:
-				case IL_META_ELEMTYPE_U1:
-				case IL_META_ELEMTYPE_BOOLEAN:
+				/* Box a primitive value after aligning it properly */
+				switch(ILType_ToElement(targetParamType))
 				{
-					boxByte = (ILInt8)(stacktop->intValue);
-					ptr = &boxByte;
-				}
-				break;
-
-				case IL_META_ELEMTYPE_I2:
-				case IL_META_ELEMTYPE_U2:
-				case IL_META_ELEMTYPE_CHAR:
-				{
-					boxShort = (ILInt16)(stacktop->intValue);
-					ptr = &boxShort;
-				}
-				break;
-
-			#ifdef IL_CONFIG_FP_SUPPORTED
-				case IL_META_ELEMTYPE_R4:
-				{
-					boxFloat = (ILFloat)(ReadFloat(stacktop));
-					ptr = &boxFloat;
-				}
-				break;
-
-				case IL_META_ELEMTYPE_R8:
-				case IL_META_ELEMTYPE_R:
-				{
-					boxDouble = (ILDouble)(ReadFloat(stacktop));
-					ptr = &boxDouble;
-				}
-				break;
-			#else
-				case IL_META_ELEMTYPE_R4:
-				case IL_META_ELEMTYPE_R8:
-				case IL_META_ELEMTYPE_R:
-				{
-					/* No FP support, so pass a "null" instead */
-					boxPtr = 0;
-					ptr = &boxPtr;
-				}
-				break;
-			#endif
-
-				default:
-				{
-					ptr = stacktop;
-				}
-				break;
-			}
-			*posn = ILExecThreadBox(thread, paramType, ptr);
-			if(!(*posn))
-			{
-				return 0;
-			}
-		}
-		else if(ILType_IsValueType(paramType))
-		{
-			/* Box value types after aligning small enumerated types */
-			ptr = stacktop;
-			if(ILTypeIsEnum(paramType))
-			{
-				enumType = ILTypeGetEnumType(paramType);
-				if(ILType_IsPrimitive(enumType))
-				{
-					switch(ILType_ToElement(enumType))
+					case IL_META_ELEMTYPE_I1:
+					case IL_META_ELEMTYPE_U1:
+					case IL_META_ELEMTYPE_BOOLEAN:
 					{
-						case IL_META_ELEMTYPE_I1:
-						case IL_META_ELEMTYPE_U1:
-						case IL_META_ELEMTYPE_BOOLEAN:
-						{
-							boxByte = (ILInt8)(stacktop->intValue);
-							ptr = &boxByte;
-						}
-						break;
+						boxByte = (ILInt8)(*(ILInt32 *)(stacktop->ptrValue));
+						ptr = &boxByte;
+					}
+					break;
 
-						case IL_META_ELEMTYPE_I2:
-						case IL_META_ELEMTYPE_U2:
-						case IL_META_ELEMTYPE_CHAR:
+					case IL_META_ELEMTYPE_I2:
+					case IL_META_ELEMTYPE_U2:
+					case IL_META_ELEMTYPE_CHAR:
+					{
+						boxShort = (ILInt16)(*(ILInt32 *)(stacktop->ptrValue));
+						ptr = &boxShort;
+					}
+					break;
+
+				#ifdef IL_CONFIG_FP_SUPPORTED
+					case IL_META_ELEMTYPE_R4:
+					{
+						boxFloat = (ILFloat)(ReadFloat(stacktop->ptrValue));
+						ptr = &boxFloat;
+					}
+					break;
+
+					case IL_META_ELEMTYPE_R8:
+					case IL_META_ELEMTYPE_R:
+					{
+						boxDouble = (ILDouble)(ReadDouble(stacktop->ptrValue));
+						ptr = &boxDouble;
+					}
+					break;
+				#else
+					case IL_META_ELEMTYPE_R4:
+					case IL_META_ELEMTYPE_R8:
+					case IL_META_ELEMTYPE_R:
+					{
+						/* No FP support, so pass a "null" instead */
+						boxPtr = 0;
+						ptr = &boxPtr;
+					}
+					break;
+				#endif
+
+					default:
+					{
+						ptr = stacktop->ptrValue;
+					}
+					break;
+				}
+				*posn = ILExecThreadBox(thread, targetParamType, ptr);
+				if(!(*posn))
+				{
+					return 0;
+				}
+			}
+			else if(ILType_IsValueType(targetParamType))
+			{
+				/* Box value types after aligning small enumerated types */
+				ptr = stacktop->ptrValue;
+				if(ILTypeIsEnum(targetParamType))
+				{
+					enumType = ILTypeGetEnumType(targetParamType);
+					if(ILType_IsPrimitive(enumType))
+					{
+						switch(ILType_ToElement(enumType))
 						{
-							boxShort = (ILInt16)(stacktop->intValue);
-							ptr = &boxShort;
+							case IL_META_ELEMTYPE_I1:
+							case IL_META_ELEMTYPE_U1:
+							case IL_META_ELEMTYPE_BOOLEAN:
+							{
+								boxByte = (ILInt8)(*(ILInt32 *)(stacktop->ptrValue));
+								ptr = &boxByte;
+							}
+							break;
+
+							case IL_META_ELEMTYPE_I2:
+							case IL_META_ELEMTYPE_U2:
+							case IL_META_ELEMTYPE_CHAR:
+							{
+								boxShort = (ILInt16)(*(ILInt32 *)(stacktop->ptrValue));
+								ptr = &boxShort;
+							}
+							break;
 						}
-						break;
 					}
 				}
+				*posn = ILExecThreadBox(thread, targetParamType, ptr);
+				if(!(*posn))
+				{
+					return 0;
+				}
 			}
-			*posn = ILExecThreadBox(thread, paramType, ptr);
-			if(!(*posn))
+			else if(ILTypeIsReference(targetParamType))
 			{
-				return 0;
+				/* Ref to an object reference type: pass the object reference */
+				*posn = *((ILObject **)(stacktop->ptrValue));
+			}
+			else
+			{
+				/* Assume that everything else is a pointer, and wrap
+				it up within a "System.IntPtr" object */
+				*posn = ILExecThreadBox(thread, ILType_Int, stacktop->ptrValue);
+				if(!(*posn))
+				{
+					return 0;
+				}
 			}
-		}
-		else if(ILTypeIsReference(paramType))
-		{
-			/* Object reference type: pass it directly */
-			*posn = (ILObject *)(stacktop->ptrValue);
 		}
 		else
 		{
-			/* Assume that everything else is a pointer, and wrap
-			   it up within a "System.IntPtr" object */
-			*posn = ILExecThreadBox(thread, ILType_Int, stacktop);
-			if(!(*posn))
+			if(ILType_IsPrimitive(paramType))
 			{
-				return 0;
+				int x;
+
+				x = ILType_ToElement(paramType);
+				/* Box a primitive value after aligning it properly */
+				switch(ILType_ToElement(paramType))
+				{
+					case IL_META_ELEMTYPE_I1:
+					case IL_META_ELEMTYPE_U1:
+					case IL_META_ELEMTYPE_BOOLEAN:
+					{
+						boxByte = (ILInt8)(stacktop->intValue);
+						ptr = &boxByte;
+					}
+					break;
+
+					case IL_META_ELEMTYPE_I2:
+					case IL_META_ELEMTYPE_U2:
+					case IL_META_ELEMTYPE_CHAR:
+					{
+						boxShort = (ILInt16)(stacktop->intValue);
+						ptr = &boxShort;
+					}
+					break;
+
+				#ifdef IL_CONFIG_FP_SUPPORTED
+					case IL_META_ELEMTYPE_R4:
+					{
+						boxFloat = (ILFloat)(ReadFloat(stacktop));
+						ptr = &boxFloat;
+					}
+					break;
+
+					case IL_META_ELEMTYPE_R8:
+					case IL_META_ELEMTYPE_R:
+					{
+						boxDouble = (ILDouble)(ReadDouble(stacktop));
+						ptr = &boxDouble;
+					}
+					break;
+				#else
+					case IL_META_ELEMTYPE_R4:
+					case IL_META_ELEMTYPE_R8:
+					case IL_META_ELEMTYPE_R:
+					{
+						/* No FP support, so pass a "null" instead */
+						boxPtr = 0;
+						ptr = &boxPtr;
+					}
+					break;
+				#endif
+
+					default:
+					{
+						ptr = stacktop;
+					}
+					break;
+				}
+				*posn = ILExecThreadBox(thread, paramType, ptr);
+				if(!(*posn))
+				{
+					return 0;
+				}
+			}
+			else if(ILType_IsValueType(paramType))
+			{
+				/* Box value types after aligning small enumerated types */
+				ptr = stacktop;
+				if(ILTypeIsEnum(paramType))
+				{
+					enumType = ILTypeGetEnumType(paramType);
+					if(ILType_IsPrimitive(enumType))
+					{
+						switch(ILType_ToElement(enumType))
+						{
+							case IL_META_ELEMTYPE_I1:
+							case IL_META_ELEMTYPE_U1:
+							case IL_META_ELEMTYPE_BOOLEAN:
+							{
+								boxByte = (ILInt8)(stacktop->intValue);
+								ptr = &boxByte;
+							}
+							break;
+
+							case IL_META_ELEMTYPE_I2:
+							case IL_META_ELEMTYPE_U2:
+							case IL_META_ELEMTYPE_CHAR:
+							{
+								boxShort = (ILInt16)(stacktop->intValue);
+								ptr = &boxShort;
+							}
+							break;
+						}
+					}
+				}
+				*posn = ILExecThreadBox(thread, paramType, ptr);
+				if(!(*posn))
+				{
+					return 0;
+				}
+			}
+			else if(ILTypeIsReference(paramType))
+			{
+				/* Object reference type: pass it directly */
+				*posn = (ILObject *)(stacktop->ptrValue);
+			}
+			else
+			{
+				/* Assume that everything else is a pointer, and wrap
+				it up within a "System.IntPtr" object */
+				*posn = ILExecThreadBox(thread, ILType_Int, stacktop);
+				if(!(*posn))
+				{
+					return 0;
+				}
 			}
 		}
-		stacktop += StackWordsForType(thread, paramType);
+
+		stacktop += _ILStackWordsForType(thread, paramType);
 		++posn;
 	}
 
@@ -363,7 +529,7 @@ ILUInt32 _ILGetMethodParamCount(ILExecThread *thread, ILMethod *method,
 	numParams = ILTypeNumParams(signature);
 	for(param = 1; param <= numParams; ++param)
 	{
-		num += StackWordsForType(thread, ILTypeGetParam(signature, param));
+		num += _ILStackWordsForType(thread, ILTypeGetParam(signature, param));
 	}
 
 	/* Account for the vararg array parameter */
@@ -594,6 +760,8 @@ VMBREAK(COP_CALL_CTOR);
  */
 VMCASE(COP_CALL_NATIVE):
 {
+	BEGIN_NATIVE_CALL();
+
 	/* Call a native method */
 	COPY_STATE_TO_THREAD();
 	FFI_CALL(CVM_ARG_PTR2(void *), CVM_ARG_PTR(void *),
@@ -601,6 +769,8 @@ VMCASE(COP_CALL_NATIVE):
 	RESTORE_STATE_FROM_THREAD();
 	pc = thread->pc;
 	MODIFY_PC_AND_STACK(CVM_LEN_PTR2, -1);
+
+	END_NATIVE_CALL();
 }
 VMBREAK(COP_CALL_NATIVE);
 
@@ -620,12 +790,16 @@ VMBREAK(COP_CALL_NATIVE);
  */
 VMCASE(COP_CALL_NATIVE_VOID):
 {
+	BEGIN_NATIVE_CALL();
+
 	/* Call a native method that has no return value */
 	COPY_STATE_TO_THREAD();
 	FFI_CALL(CVM_ARG_PTR2(void *), CVM_ARG_PTR(void *), 0, nativeArgs);
 	RESTORE_STATE_FROM_THREAD();
 	pc = thread->pc;
 	MODIFY_PC_AND_STACK(CVM_LEN_PTR2, 0);
+
+	END_NATIVE_CALL();
 }
 VMBREAK(COP_CALL_NATIVE_VOID);
 
@@ -657,6 +831,8 @@ VMBREAK(COP_CALL_NATIVE_VOID);
  */
 VMCASE(COP_CALL_NATIVE_RAW):
 {
+	BEGIN_NATIVE_CALL();
+
 	/* Call a native method using the raw API */
 	COPY_STATE_TO_THREAD();
 	FFI_RAW_CALL(CVM_ARG_PTR2(void *), CVM_ARG_PTR(void *),
@@ -664,6 +840,8 @@ VMCASE(COP_CALL_NATIVE_RAW):
 	RESTORE_STATE_FROM_THREAD();
 	pc = thread->pc;
 	MODIFY_PC_AND_STACK(CVM_LEN_PTR2, -2);
+
+	END_NATIVE_CALL();
 }
 VMBREAK(COP_CALL_NATIVE_RAW);
 
@@ -687,6 +865,8 @@ VMBREAK(COP_CALL_NATIVE_RAW);
  */
 VMCASE(COP_CALL_NATIVE_VOID_RAW):
 {
+	BEGIN_NATIVE_CALL();
+
 	/* Call a native method that has no return value using the raw API */
 	COPY_STATE_TO_THREAD();
 	FFI_RAW_CALL(CVM_ARG_PTR2(void *), CVM_ARG_PTR(void *),
@@ -694,6 +874,8 @@ VMCASE(COP_CALL_NATIVE_VOID_RAW):
 	RESTORE_STATE_FROM_THREAD();
 	pc = thread->pc;
 	MODIFY_PC_AND_STACK(CVM_LEN_PTR2, -1);
+
+	END_NATIVE_CALL();
 }
 VMBREAK(COP_CALL_NATIVE_VOID_RAW);
 
@@ -972,6 +1154,9 @@ VMCASE(COP_RETURN):
 	/* Return from a method with no return value */
 	stacktop = frame;
 popFrame:
+
+	CHECK_ABORT();
+
 	callFrame = &(thread->frameStack[--(thread->numFrames)]);
 	methodToCall = callFrame->method;
 	pc = callFrame->pc;
@@ -1751,7 +1936,7 @@ VMCASE(COP_PREFIX_PACK_VARARGS):
 	/* Pack a set of arguments for a vararg method call */
 #ifdef IL_CONFIG_VARARGS
 	COPY_STATE_TO_THREAD();
-	tempNum = PackVarArgs(thread, stacktop, CVMP_ARG_WORD, CVMP_ARG_WORD2,
+	tempNum = _ILPackVarArgs(thread, stacktop, CVMP_ARG_WORD, CVMP_ARG_WORD2,
 						  CVMP_ARG_WORD2_PTR(ILType *), &tempptr);
 	RESTORE_STATE_FROM_THREAD();
 	stacktop -= tempNum;
@@ -1781,7 +1966,7 @@ VMBREAK(COP_PREFIX_PACK_VARARGS);
  */
 VMCASE(COP_PREFIX_PROFILE_COUNT):
 {
-	++(method->count);
+	ILInterlockedIncrement(&method->count);
 	MODIFY_PC_AND_STACK(CVMP_LEN_NONE,0);
 }
 VMBREAK(COP_PREFIX_PROFILE_COUNT);
@@ -1810,4 +1995,100 @@ VMCASE(COP_PREFIX_WADDR_NATIVE_N):
 }
 VMBREAK(COP_PREFIX_WADDR_NATIVE_N);
 
+/**
+ *<opcode name="trace_in" group="Profiling Instructions">
+ *	<operation>Print the name of the called method . This is injected
+ *             at the top of every method in --trace mode</operation>
+ *
+ * 	<format>trace_in<fsep/>reason</format>
+ * 	<dformat>{trace_in}<fsep/>reason</dformat>
+ *
+ * 	<form name="trace_in" code="COP_PREFIX_TRACE_IN"/>
+ *
+ *  <description>This instruction prints out the called method .  
+ *  It is normally inserted immediately before a method call when 
+ *  the engine is running in --trace mode. The reason parameter
+ *  is provided for future additions , and is ignored in the current
+ *  implementation.
+ *  </description>
+ * </opcode>
+ */
+VMCASE(COP_PREFIX_TRACE_IN):
+{
+#if !defined(IL_CONFIG_REDUCE_CODE) && !defined(IL_WITHOUT_TOOLS)
+	ILMutexLock(globalTraceMutex);
+#ifdef INDENT_TRACE
+	int depth = thread->numFrames;
+	while(depth--) 
+	{
+		putc(' ',stdout); 
+	}
+#endif
+	fputs("Entering ", stdout);
+	ILDumpMethodType(stdout, ILProgramItem_Image(method),
+					 ILMethod_Signature(method), 0,
+					 ILMethod_Owner(method),
+					 ILMethod_Name(method), method);
+	putc('\n',stdout);
+	fflush(stdout);
+	ILMutexUnlock(globalTraceMutex);
+#endif
+	MODIFY_PC_AND_STACK(CVMP_LEN_WORD, 0);
+}
+VMBREAK(COP_PREFIX_TRACE_IN);
+
+/**
+ *<opcode name="trace_out" group="Profiling Instructions">
+ *	<operation>Print the name of the callee method while returning to it. 
+ *             This is injected for every return in --trace mode</operation>
+ *
+ * 	<format>trace_out<fsep/>reason</format>
+ * 	<dformat>{trace_out}<fsep/>reason</dformat>
+ *
+ * 	<form name="trace_out" code="COP_PREFIX_TRACE_OUT"/>
+ *
+ *  <description>This instruction prints out the callee method 
+ *  on returning from the current method.  It is normally inserted 
+ *  immediately before a return from a method when the engine is 
+ *  running in --trace mode.
+ *  </description>
+ * </opcode>
+ */
+
+VMCASE(COP_PREFIX_TRACE_OUT):
+{
+#if !defined(IL_CONFIG_REDUCE_CODE) && !defined(IL_WITHOUT_TOOLS)
+	// NOTE: at some point of time use the Reason parameter to
+	// carry more information about this 
+	ILMethod * methodToReturn = ILExecThreadStackMethod(thread, 1);
+	ILMutexLock(globalTraceMutex);
+#ifdef INDENT_TRACE
+	int depth = thread->numFrames;
+	while(depth--) 
+	{
+		putc(' ',stdout); 
+	}
+#endif
+	fputs("Returning to ", stdout);
+	if(methodToReturn)
+	{
+		ILDumpMethodType(stdout, ILProgramItem_Image(methodToReturn),
+						 ILMethod_Signature(methodToReturn), 0,
+						 ILMethod_Owner(methodToReturn),
+						 ILMethod_Name(methodToReturn), methodToReturn);
+	}
+	else
+	{
+		fputs(" (ilrun_main) ",stdout);
+	}
+	putc('\n',stdout);
+	fflush(stdout);
+	ILMutexUnlock(globalTraceMutex);
+#endif
+	MODIFY_PC_AND_STACK(CVMP_LEN_WORD, 0);
+}
+VMBREAK(COP_PREFIX_TRACE_OUT);
+
+
+
 #endif /* IL_CVM_PREFIX */
