@@ -59,6 +59,11 @@ extern	"C" {
 	/*#define	IL_USE_IMTS	1*/
 #endif
 
+#if !defined(IL_CONFIG_REDUCE_CODE) && !defined(IL_WITHOUT_TOOLS)
+/* Global lock for trace outputs */
+extern ILMutex *globalTraceMutex;
+#endif
+
 /*
  * Structure that keeps track of a loaded external module.
  */
@@ -149,9 +154,9 @@ struct _tagILExecProcess
 	/* Hash table that contains all intern'ed strings within the system */
 	void		   *internHash;
 
-#ifdef USE_HASHING_MONITORS
+#ifdef IL_CONFIG_USE_THIN_LOCKS
 	/* Hash table that contains all monitors */
-	void			*monitorHash;
+	void			*monitorTable;
 	ILMutex		*monitorSystemLock;
 #endif
 
@@ -248,8 +253,10 @@ struct _tagILExecThread
 	/* Last exception that was thrown */
 	ILObject       *thrownException;
 
-	/* Indicates if an abort is in progress */
-	int				aborting;
+	/* Flag that indicates whether a thread is aborting */
+	volatile int	aborting;
+	/* Flag that indicates whether an abort has been requested */
+	volatile int	abortRequested;
 
 	/* System.Threading.Thread object */
 	ILObject *clrThread;
@@ -261,6 +268,9 @@ struct _tagILExecThread
 	/* Free monitors list */
 	ILExecMonitor *freeMonitor;
 
+	/* Number of monitors in the free monitor list */
+	int freeMonitorCount;
+
 	/* Stack of call frames in use */
 	ILCallFrame	   *frameStack;
 	ILUInt32		numFrames;
@@ -270,6 +280,21 @@ struct _tagILExecThread
 	void		  **threadStaticSlots;
 	ILUInt32		threadStaticSlotsUsed;
 
+	/* Flagged if the thread is running managed code */
+	int		runningManagedCode;
+
+	/* The last exception that was thrown (as seen from the CVM)
+	   This always stores the last exception thrown and is never reset */
+	ILObject	*currentException;
+
+	/* The ThreadAbortException instance of the thread is being aborted */
+	ILObject	*threadAbortException;
+
+	/* The PC where the ThreadAbortException should be rethrown */
+	unsigned char	*abortHandlerEndPC;
+
+	/* The frame where the ThreadAbortException was first noticed */
+	ILUInt32		abortHandlerFrame;
 };
 
 /*
@@ -312,6 +337,7 @@ struct _tagILClassPrivate
 	ILObject       *clrType;			/* Associated CLR type object */
 	ILObject       *staticData;			/* Static data area object */
 	ILImplPrivate  *implements;			/* Interface implementation records */
+	ILNativeInt		gcTypeDescriptor;	/* Describes the layout of the type for the GC */
 #ifdef IL_USE_IMTS
 	ILUInt32		imtBase;			/* Base for IMT identifiers */
 	ILMethod	   *imt[IL_IMT_SIZE];	/* Interface method table */
@@ -323,8 +349,8 @@ typedef void * ILLockWord;
 
 struct _tagILExecMonitor
 {
-	ILWaitHandle *supportMonitor;
-	volatile ILInt32 waiters;
+	ILWaitHandle *waitMutex;
+	volatile ILInt32 waiters;	
 	ILExecMonitor *next;
 };
 
@@ -336,10 +362,10 @@ typedef struct _tagObjectHeader ILObjectHeader;
 struct _tagObjectHeader
 {
 	ILClassPrivate *classPrivate;
-#ifdef USE_HASHING_MONITORS
+#ifdef IL_CONFIG_USE_THIN_LOCKS
 	/* NOTHING */
 #else
-	ILLockWord lockWord;
+	volatile ILLockWord lockWord;
 #endif
 };
 
@@ -669,19 +695,34 @@ ILObject *_ILCustomToObject(ILExecThread *thread, void *ptr,
 							const char *customCookie, int customCookieLen);
 
 /*
- *	Gets the current managed thread object from an engine thread.
+ *	Gets the managed thread object from an engine thread.
  */
-ILObject *_ILGetCurrentClrThread(ILExecThread *thread);
+ILObject *ILExecThreadGetClrThread(ILExecThread *thread);
 
 /*
- *	Makes the given support thread execute in the context of the given engine thread.
+ * Associates a support thread with an engine thread and the engine
+ * thread with the support thread.
  */
 void _ILThreadExecuteOn(ILThread *thread, ILExecThread *execThread);
 
 /*
- *	Throws a thread abort exception on the given thread.
+ * Disassociates a support thread with an engine thread and the
+ * engine thread with the support thread.
+ * The support thread will no longer be able to execute managed
+ * code and the engine thread will no longer be able to be used
+ * to execute anything until it is reassociated using ILThreadExecuteOn.
  */
-void _ILExecThreadThrowThreadAbortException(ILExecThread *thread, ILObject *stateInfo);
+void _ILThreadUnexecuteOn(ILThread *thread, ILExecThread *execThread);
+
+/*
+ *	Constructs and returns a new ThreadAbortException for the given thread.
+ */
+ILObject *_ILExecThreadNewThreadAbortException(ILExecThread *thread, ILObject *stateInfo);
+
+/*
+ * Returns 1 if the current exception is a ThreadAbortException.
+ */
+int _ILExecThreadCurrentExceptionThreadIsAbortException(ILExecThread *thread);
 
 /*
  *	Aborts the current thread.
@@ -694,14 +735,36 @@ void _ILAbortThread(ILExecThread *thread);
 void _ILHandleWaitResult(ILExecThread *thread, int result);
 
 /*
- *	Creates a monitor used by the execution engine.
+ * Creates a monitor used by the execution engine.
+ * The monitor is created on the GC heap and will be automatically
+ * collected.
  */
-ILExecMonitor *ILExecMonitorCreate(void);
+ILExecMonitor *_ILExecMonitorCreate(void);
 
 /*
- *	Destroys a monitor used by the execution engine.
+ * Intializes the monitor system for the given process.
  */
-void ILExecMonitorDestroy(ILExecMonitor *monitor);
+int _ILExecMonitorProcessCreate(ILExecProcess *process);
+
+/*
+ * Cleans up the monitor system for the given process.
+ */
+int _ILExecMonitorProcessDestroy(ILExecProcess *process);
+
+/*
+ * Pack a set of arguments into a vararg "Object[]" array.
+ * Returns the number of stack words to pop from the function,
+ * and the new array in "*array".
+ */
+ILUInt32 _ILPackVarArgs(ILExecThread *thread, CVMWord *stacktop,
+							ILUInt32 firstParam, ILUInt32 numArgs,
+							ILType *callSiteSig, void **array);
+
+/*
+ * Determine the number of stack words that are occupied
+ * by a specific type.
+ */
+ILUInt32 _ILStackWordsForType(ILExecThread *thread, ILType *type);
 
 #ifdef	__cplusplus
 };
