@@ -61,13 +61,50 @@ static int InitializeClass(ILExecThread *thread, ILClass *classInfo)
 	return 1;
 }
 
+static IL_INLINE ILMethod *FindFinalizeMethod(ILClassPrivate *classPrivate)
+{
+	ILMethod *method;
+	const char *methodName;
+	ILType *signature;
+	ILClass *classInfo;
+	
+	if (!classPrivate->hasFinalizer)
+	{
+		return 0;
+	}
+
+	classInfo = classPrivate->classInfo;
+
+	while(classInfo != 0)
+	{
+		method = 0;
+		while((method = (ILMethod *)ILClassNextMemberByKind
+			(classInfo, (ILMember *)method, IL_META_MEMBERKIND_METHOD)) != 0)
+		{
+			methodName = ILMethod_Name(method);
+
+			if(methodName[0] == 'F' && strcmp(methodName, "Finalize") == 0)
+			{
+				signature = ILMethod_Signature(method);
+
+				if(ILTypeGetReturn(signature) == ILType_Void &&
+				   ILTypeNumParams(signature) == 0)
+				{
+					return method;
+				}
+			}
+		}
+		classInfo = ILClassGetParent(classInfo);
+	}
+
+	return 0;
+}
+
 /* This method assumes there is only one finalizer thread for the entire os process */
 void _ILFinalizeObject(void *block, void *data)
 {
 	ILObject *object;
-	ILClass *classInfo;
 	ILMethod *method;
-	ILType *signature;
 	ILThread *thread;
 	ILExecProcess *process;
 	ILExecThread *execThread;
@@ -96,7 +133,7 @@ void _ILFinalizeObject(void *block, void *data)
 	thread = ILThreadSelf();
 
 	if (execThread == 0)
-	{				
+	{
 		/* Create a new engine thread for the finalizers of this process to run on */
 
 		execThread = _ILExecThreadCreate(process);
@@ -106,47 +143,29 @@ void _ILFinalizeObject(void *block, void *data)
 			return;
 		}
 
-		execThread->supportThread = thread;
 		process->finalizerThread = execThread;
 	}
 
-	/* Make the finalizer thread execute in the context of the object's process's
-	    finalizer thread */
+	/* Make the finalizer thread execute in the context of the object's
+	   process's finalizer thread which could actually be the main thread
+	   Don't call ILThreadUnexecuteOn afterwards because single-threaded 
+	   systems share the main thread with the GC finalizer and we don't 
+	   want to make the ILExecThread for the main thread unusable */
 	_ILThreadExecuteOn(thread, execThread);
 
-	/* Get the object's class and locate the "Finalize" method */
-	classInfo = GetObjectClass(object);
+	method = FindFinalizeMethod(GetObjectClassPrivate(object));
 
-	while(classInfo != 0)
+	if (method != 0)
 	{
-		method = 0;
-		while((method = (ILMethod *)ILClassNextMemberByKind
-			(classInfo, (ILMember *)method, IL_META_MEMBERKIND_METHOD)) != 0)
-		{
-			if(!strcmp(ILMethod_Name(method), "Finalize"))
-			{
-				signature = ILMethod_Signature(method);
-				if(ILTypeGetReturn(signature) == ILType_Void &&
-				   ILTypeNumParams(signature) == 0)
-				{
-					ILGCRegisterFinalizer(block, 0, 0);
+		ILGCRegisterFinalizer(block, 0, 0);
 		
-					/* Invoke the "Finalize" method on the object */
-					if(ILExecThreadCall(ILExecThreadCurrent(),
-										method, (void *)0, object))
-					{
-						ILExecThreadClearException(ILExecThreadCurrent());
-					}
-				}
-			}
+		/* Invoke the "Finalize" method on the object */
+		if(ILExecThreadCall(ILExecThreadCurrent(),
+							method, (void *)0, object))
+		{
+			ILExecThreadClearException(ILExecThreadCurrent());
 		}
-		classInfo = ILClassGetParent(classInfo);
 	}
-
-	execThread->supportThread = 0;
-
-	/* Make sure the finalizer thread can no longer execute managed code */
-	_ILThreadExecuteOn(thread, 0);	
 }
 
 ILObject *_ILEngineAlloc(ILExecThread *thread, ILClass *classInfo,
@@ -155,43 +174,45 @@ ILObject *_ILEngineAlloc(ILExecThread *thread, ILClass *classInfo,
 	void *ptr;
 	ILObject *obj;
 
-	/* Make sure the class has been initialized before we start */
-	if(classInfo != 0 && !InitializeClass(thread, classInfo))
+	if (classInfo == 0)
 	{
-		return 0;
-	}
-
-	/* Allocate memory from the heap */
-	ptr = ILGCAlloc(size + IL_OBJECT_HEADER_SIZE);
-	if(!ptr)
-	{
-		/* Throw an "OutOfMemoryException" */
-		thread->thrownException = thread->process->outOfMemoryObject;
-		return 0;
-	}
-
-	obj = GetObjectFromGcBase(ptr);
-
-	/* Set the class into the block */
-	if(classInfo)
-	{
-		SetObjectClassPrivate(obj, (ILClassPrivate *)(classInfo->userData));
+		/* Allocating non-object memory so no need to make space for the header. */
+		return ILGCAlloc(size);
 	}
 	else
 	{
-		SetObjectClassPrivate(obj, 0);
-	}
+		/* Make sure the class has been initialized before we start */
+		if (!InitializeClass(thread, classInfo))
+		{
+			return 0;
+		}
 
-	/* Attach a finalizer to the object if the class has
-	   a non-trival finalizer method attached to it */
-	if(classInfo != 0 &&
-	   ((ILClassPrivate *)(classInfo->userData))->hasFinalizer)
-	{
-		ILGCRegisterFinalizer(ptr, _ILFinalizeObject, thread->process->finalizationContext);
-	}
+		/* Allocate memory from the heap */
+		ptr = ILGCAlloc(size + IL_OBJECT_HEADER_SIZE);
+		
+		if(!ptr)
+		{
+			/* Throw an "OutOfMemoryException" */
+			thread->thrownException = thread->process->outOfMemoryObject;
+			return 0;
+		}
 
-	/* Return a pointer to the object */
-	return obj;
+		obj = GetObjectFromGcBase(ptr);
+
+		/* Set the class into the block */
+		SetObjectClassPrivate(obj, (ILClassPrivate *)(classInfo->userData));
+		
+
+		/* Attach a finalizer to the object if the class has
+		a non-trival finalizer method attached to it */
+		if(((ILClassPrivate *)(classInfo->userData))->hasFinalizer)
+		{
+			ILGCRegisterFinalizer(ptr, _ILFinalizeObject, thread->process->finalizationContext);
+		}
+
+		/* Return a pointer to the object */
+		return obj;
+	}
 }
 
 ILObject *_ILEngineAllocAtomic(ILExecThread *thread, ILClass *classInfo,
@@ -199,44 +220,58 @@ ILObject *_ILEngineAllocAtomic(ILExecThread *thread, ILClass *classInfo,
 {
 	void *ptr;
 	ILObject *obj;
+	ILClassPrivate *classPrivate;
 
-	/* Make sure the class has been initialized before we start */
-	if(classInfo != 0 && !InitializeClass(thread, classInfo))
+	if (classInfo == 0)
 	{
-		return 0;
-	}
-
-	/* Allocate memory from the heap */
-	ptr = ILGCAllocAtomic(size + IL_OBJECT_HEADER_SIZE);	
-	if(!ptr)
-	{
-		/* Throw an "OutOfMemoryException" */
-		thread->thrownException = thread->process->outOfMemoryObject;
-		return 0;
-	}
-	
-	obj = GetObjectFromGcBase(ptr);
-
-	/* Set the class into the block */
-	if(classInfo)
-	{
-		SetObjectClassPrivate(obj, (ILClassPrivate *)(classInfo->userData));
+		/* Allocating non-object memory so no need to make space for the header. */
+		return ILGCAllocAtomic(size);
 	}
 	else
 	{
-		SetObjectClassPrivate(obj, 0);
-	}
+		/* Make sure the class has been initialized before we start */
+		if(!InitializeClass(thread, classInfo))
+		{
+			return 0;
+		}
 
-	/* Attach a finalizer to the object if the class has
-	   a non-trival finalizer method attached to it */
-	if(classInfo != 0 &&
-	   ((ILClassPrivate *)(classInfo->userData))->hasFinalizer)
-	{
-		ILGCRegisterFinalizer(ptr, _ILFinalizeObject, thread->process->finalizationContext);
-	}
+#ifdef IL_CONFIG_USE_THIN_LOCKS
+		/* Allocate memory from the heap */
+		ptr = ILGCAllocAtomic(size + IL_OBJECT_HEADER_SIZE);
+#else
+		classPrivate = (ILClassPrivate *)(classInfo->userData);
 
-	/* Return a pointer to the object */
-	return obj;
+		/* TODO: Move descriptor creation to layout.c */
+		if (classPrivate->gcTypeDescriptor == IL_MAX_NATIVE_UINT)
+		{
+			ILNativeUInt bitmap = IL_OBJECT_HEADER_PTR_MAP;
+
+			classPrivate->gcTypeDescriptor = ILGCCreateTypeDescriptor(&bitmap, IL_OBJECT_HEADER_SIZE / sizeof(ILNativeInt));		
+		}
+
+		ptr = ILGCAllocExplicitlyTyped(size + IL_OBJECT_HEADER_SIZE, classPrivate->gcTypeDescriptor);
+#endif
+		if(!ptr)
+		{
+			/* Throw an "OutOfMemoryException" */
+			thread->thrownException = thread->process->outOfMemoryObject;
+			return 0;
+		}
+		
+		obj = GetObjectFromGcBase(ptr);
+
+		SetObjectClassPrivate(obj, (ILClassPrivate *)(classInfo->userData));
+		
+		/* Attach a finalizer to the object if the class has
+		   a non-trival finalizer method attached to it */
+		if(((ILClassPrivate *)(classInfo->userData))->hasFinalizer)
+		{
+			ILGCRegisterFinalizer(ptr, _ILFinalizeObject, thread->process->finalizationContext);
+		}
+
+		/* Return a pointer to the object */
+		return obj;
+	}
 }
 
 ILObject *_ILEngineAllocObject(ILExecThread *thread, ILClass *classInfo)
