@@ -1053,14 +1053,113 @@ ILObject *_IL_Assembly_GetManifestResourceInfo(ILExecThread *thread,
 	return 0;
 }
 
+/*
+ * Walk through all resources available, assemlbing their names
+ * into the passed array.  Return the number of resources found.
+ */
+static ILInt32 WalkResourceNames(ILExecThread *thread, ILImage *image, ILString** buf)
+{
+	ILInt32 count = 0;
+	ILManifestRes *res = 0;
+	ILFileDecl *file;
+
+	res = 0;
+	while((res = (ILManifestRes *)ILImageNextToken
+			(image, IL_META_TOKEN_MANIFEST_RESOURCE, (void *)res)) != 0)
+	{
+		/* TODO: handle resources in external assemblies */
+		if(ILManifestRes_OwnerAssembly(res))
+		{
+			continue;
+		}
+
+		/* If the resource is private, and the caller is not
+			the same image, then skip it */
+		if(!ILManifestRes_IsPublic(res) && _ILClrCallerImage(thread) != image)
+		{
+			continue;
+		}
+
+		/* Handle resources in external files */
+		if((file = ILManifestRes_OwnerFile(res)) != 0)
+		{
+			/* If the calling image is not secure, then skip it.
+				Otherwise, applications may be able to force
+				the engine to read arbitrary files from the
+				same directory as a library assembly */
+			if(!ILImageIsSecure(_ILClrCallerImage(thread)))
+			{
+				continue;
+			}
+
+			/* The file must not have metadata and it must
+				start at offset 0 to be a valid file resource */
+			if(ILFileDecl_HasMetaData(file) ||
+				ILManifestRes_Offset(res) != 0)
+			{
+				continue;
+			}
+		}
+
+		/* Record the name in the return buffer */
+		if(buf != 0)
+		{
+			buf[count] = ILStringCreate(thread, ILManifestRes_Name(res));
+			if(!(buf[count]))
+			{
+				/* Out of memory exception thrown */
+				return -1;
+			}
+		}
+		++count;
+	}
+
+	return count;
+}
+
 /*
  * public virtual String[] GetManifestResourceNames();
  */
 System_Array *_IL_Assembly_GetManifestResourceNames(ILExecThread *thread,
 													ILObject *_this)
 {
-	/* TODO */
-	return 0;
+	ILInt32 arraySize;
+	ILString** buf;
+	System_Array *files;
+	ILImage *image;
+	ILProgramItem *item;
+
+	/* Get the image that corresponds to the assembly */
+	item = (ILProgramItem *)_ILClrFromObject(thread, _this);
+	if(!item)
+	{
+	  	return 0;
+	}
+	image = ILProgramItem_Image(item);
+	if(!image)
+	{
+		return 0;
+	}
+
+	/* Get the number of array elements required */
+	arraySize = WalkResourceNames(thread, image, 0);
+	if(arraySize < 0)
+	{
+		return 0;
+	}
+
+	/* Create the object we will return */
+	files = (System_Array*) ILExecThreadNew
+		(thread, "[oSystem.String;", "(Ti)V",(ILVaInt)arraySize);
+	if(!files)
+	{
+		return 0;
+	}
+	buf = (ILString**)(ArrayToBuffer(files));
+
+	/* Gather up the file names */
+	WalkResourceNames(thread, image, buf);
+	return files;
 }
 
 /*
@@ -2554,21 +2653,109 @@ static ILObject *InvokeMethod(ILExecThread *thread, ILMethod *method,
 	{
 		paramType = ILTypeGetParam(signature, paramNum + 1);
 		paramObject = ((ILObject **)ArrayToBuffer(parameters))[paramNum];
-		if(paramObject)
-		{
-			/* Non-null object supplied */
-			objectType = ILClassToType(GetObjectClass(paramObject));
-			if(!ILTypeAssignCompatible(image, objectType, paramType))
+
+		/* Handle byref params differently */
+		if ((ILType_IsComplex(paramType) && ILType_Kind(paramType) == IL_TYPE_COMPLEX_BYREF))
+		{			
+			ILType *paramRefType;
+			int isBoxableType;
+
+			/* Get the target type for the byref param */
+			paramRefType = ILType_Ref(paramType);
+
+			isBoxableType = ILType_IsPrimitive(paramRefType) || ILType_IsValueType(paramRefType);
+
+			if (paramObject)
 			{
-				ILExecThreadThrowSystem(thread, "System.ArgumentException", 0);
-				return 0;
+				/* If a non-null param was passed then make sure its type is compatible 
+				   with byref's target type */
+
+				objectType = ILClassToType(GetObjectClass(paramObject));
+				
+				if(!ILTypeAssignCompatible(image, objectType, ILType_Ref(paramType)))
+				{
+					ILExecThreadThrowSystem(thread, "System.ArgumentException", 0);
+					return 0;
+				}
+			}
+			else if (isBoxableType)
+			{
+				/* If a null param was passed and the param type is boxable then create 
+				   a new blank value type (This is MS.NET behaviour) */
+				
+				paramObject = ILExecThreadBoxNoValue(thread, paramRefType);
+
+				if(!paramObject)
+				{
+					ILExecThreadThrowOutOfMemory(thread);
+
+					return 0;
+				}
+
+				/* Assign the object to the object array */				
+				((ILObject **)ArrayToBuffer(parameters))[paramNum] = paramObject;
+			}
+			
+			if (isBoxableType)
+			{
+				/* If the param is a primitive or value type then pass a pointer to the boxed value */
+				args[argNum].ptrValue = (void *)paramObject;
+			}
+			else
+			{
+				/* If the param is any other type then pass a pointer to the object array element */
+				args[argNum].ptrValue = (void *)&((ILObject **)ArrayToBuffer(parameters))[paramNum];
+			}
+		}
+		else
+		{
+			if(paramObject)
+			{
+				/* If there's an argument then make sure it's the right type */
+
+				objectType = ILClassToType(GetObjectClass(paramObject));
+				
+				/* Make sure the type passed matches the param type */
+				if(!ILTypeAssignCompatible(image, objectType, paramType))
+				{
+					ILExecThreadThrowSystem(thread, "System.ArgumentException", 0);
+					return 0;
+				}
+			}
+			else if (ILType_IsPrimitive(paramType) || ILType_IsValueType(paramType))
+			{
+				/* If there's a null argument and the parameter is a value type
+				   then create a blank value type */
+
+				objectType = paramType;
+				
+				paramObject = ILExecThreadBoxNoValue(thread, paramType);
+
+				if (!paramObject)
+				{
+					ILExecThreadThrowOutOfMemory(thread);
+
+					return 0;
+				}
+			}			
+			else
+			{
+				/* The parameter must be an object ref */
+
+				if(!ILTypeAssignCompatible(image, 0, paramType))
+				{
+					ILExecThreadThrowSystem(thread, "System.ArgumentException", 0);
+					return 0;
+				}
+
+				args[argNum].objValue = 0;
 			}
 
 			/* Unbox the object into the argument structure */
 			paramType = ILTypeGetEnumType(paramType);
+
 			if(ILType_IsPrimitive(paramType))
-			{
-				/* Unbox primitive and enumerated types into the argument */
+			{	/* Unbox primitive and enumerated types into the argument */
 				if(!ILExecThreadUnboxFloat
 							(thread, objectType, paramObject, &(args[argNum])))
 				{
@@ -2580,8 +2767,8 @@ static ILObject *InvokeMethod(ILExecThread *thread, ILMethod *method,
 			else if(ILType_IsValueType(paramType))
 			{
 				/* Pass non-enumerated value types as a pointer
-				   into the boxed object.  The "CallMethodV"
-				   function will copy the data onto the stack */
+				into the boxed object.  The "CallMethodV"
+				function will copy the data onto the stack */
 				args[argNum].ptrValue = (void *)paramObject;
 			}
 			else if(ILType_IsClass(paramType))
@@ -2592,7 +2779,7 @@ static ILObject *InvokeMethod(ILExecThread *thread, ILMethod *method,
 			else if(paramType != 0 && ILType_IsComplex(paramType))
 			{
 				if(ILType_Kind(paramType) == IL_TYPE_COMPLEX_ARRAY ||
-				   ILType_Kind(paramType) == IL_TYPE_COMPLEX_ARRAY_CONTINUE)
+				ILType_Kind(paramType) == IL_TYPE_COMPLEX_ARRAY_CONTINUE)
 				{
 					/* Array object references are passed directly */
 					args[argNum].objValue = paramObject;
@@ -2612,16 +2799,6 @@ static ILObject *InvokeMethod(ILExecThread *thread, ILMethod *method,
 				return 0;
 			}
 		}
-		else
-		{
-			/* Null object supplied: the parameter must be an object ref */
-			if(!ILTypeAssignCompatible(image, 0, paramType))
-			{
-				ILExecThreadThrowSystem(thread, "System.ArgumentException", 0);
-				return 0;
-			}
-			args[argNum].objValue = 0;
-		}
 		++argNum;
 	}
 
@@ -2734,6 +2911,53 @@ ILObject *_IL_ClrConstructor_Invoke(ILExecThread *thread,
 	return InvokeMethod(thread, method, signature, 0, parameters, 1);
 }
 
+/*
+ * public override Object InvokeOnEmpty(Object obj,
+ * 								 BindingFlags invokeAttr, Binder binder,
+ *								 Object[] parameters, CultureInfo culture);
+ */
+ILObject *_IL_ClrConstructor_InvokeOnEmpty(ILExecThread *thread,
+									ILObject *_this,
+									ILObject * obj,
+									ILInt32 invokeAttr,
+									ILObject *binder,
+									System_Array *parameters,
+									ILObject *culture)
+{
+	ILMethod *method;
+	ILType *signature;
+
+	/* Extract the method item from the "this" object */
+	method = ILProgramItemToMethod(_ILClrFromObject(thread, _this));
+	if(!method)
+	{
+		/* Something is wrong with the object */
+		ILExecThreadThrowSystem(thread, "System.MissingMethodException", 0);
+		return 0;
+	}
+
+	/* Check that we have sufficient access credentials for the method */
+	if(!_ILClrCheckAccess(thread, 0, (ILMember *)method))
+	{
+		ILExecThreadThrowSystem
+			(thread, "System.Security.SecurityException", 0);
+		return 0;
+	}
+
+	/* We cannot use this on static constructors */
+	if(ILMethod_IsStaticConstructor(method))
+	{
+		ILExecThreadThrowSystem(thread, "System.MemberAccessException", 0);
+		return 0;
+	}
+
+	/* Get the constructor's signature */
+	signature = ILMethod_Signature(method);
+
+	/* Invoke the constructor method as a normal method */
+	return InvokeMethod(thread, method, signature, obj, parameters, 0);
+}
+
 /*
  * public override Object Invoke(Object obj, BindingFlags invokeAttr,
  *								 Binder binder, Object[] parameters,
