@@ -34,6 +34,7 @@ be useful for debugging in embedded environments.
 #include "engine.h"
 #include "lib_defs.h"
 #include "il_utils.h"
+#include "il_console.h"
 
 #ifdef	__cplusplus
 extern	"C" {
@@ -71,7 +72,20 @@ void _IL_Stdio_StdWrite_ic(ILExecThread *thread, ILInt32 fd, ILUInt16 value)
 #ifndef REDUCED_STDIO
 	switch(fd)
 	{
-		case 1:		putc((value & 0xFF), stdout); break;
+		case 1:
+		{
+			if(ILConsoleGetMode() == IL_CONSOLE_NORMAL)
+			{
+				putc((value & 0xFF), stdout);
+			}
+			else
+			{
+				ILConsoleWriteChar(value);
+				fflush(stdout);
+			}
+		}
+		break;
+
 		case 2:		putc((value & 0xFF), stderr); break;
 	}
 #else
@@ -87,11 +101,24 @@ static void StdWrite(ILInt32 fd, ILUInt16 *buf, ILInt32 length)
 #ifndef REDUCED_STDIO
 	if(fd == 1)
 	{
-		while(length > 0)
+		if(ILConsoleGetMode() == IL_CONSOLE_NORMAL)
 		{
-			putc((*buf & 0xFF), stdout);
-			++buf;
-			--length;
+			while(length > 0)
+			{
+				putc((*buf & 0xFF), stdout);
+				++buf;
+				--length;
+			}
+		}
+		else
+		{
+			while(length > 0)
+			{
+				ILConsoleWriteChar(*buf);
+				++buf;
+				--length;
+			}
+			fflush(stdout);
 		}
 	}
 	else if(fd == 2)
@@ -133,7 +160,21 @@ void _IL_Stdio_StdWrite_iaBii(ILExecThread *thread, ILInt32 fd,
 #ifndef REDUCED_STDIO
 	if(fd == 1)
 	{
-		fwrite(((ILUInt8 *)ArrayToBuffer(value)) + index, 1, count, stdout);
+		if(ILConsoleGetMode() == IL_CONSOLE_NORMAL)
+		{
+			fwrite(((ILUInt8 *)ArrayToBuffer(value)) + index, 1, count, stdout);
+		}
+		else
+		{
+			ILUInt8 *buf = ((ILUInt8 *)ArrayToBuffer(value)) + index;
+			while(count > 0)
+			{
+				ILConsoleWriteChar(*buf);
+				++buf;
+				--count;
+			}
+			fflush(stdout);
+		}
 	}
 	else if(fd == 2)
 	{
@@ -170,11 +211,13 @@ void _IL_Stdio_StdWrite_iString(ILExecThread *thread, ILInt32 fd,
  */
 ILInt32 _IL_Stdio_StdRead_i(ILExecThread *thread, ILInt32 fd)
 {
+#ifndef REDUCED_STDIO
 	if(fd == 0)
 	{
-		return (ILInt32)ILInputGetChar();
+		return (ILInt32)(getc(stdin));
 	}
 	else
+#endif
 	{
 		return -1;
 	}
@@ -187,6 +230,7 @@ ILInt32 _IL_Stdio_StdRead_iacii(ILExecThread *thread, ILInt32 fd,
 								System_Array *value, ILInt32 index,
 								ILInt32 count)
 {
+#ifndef REDUCED_STDIO
 	ILUInt16 *buf = ((ILUInt16 *)(ArrayToBuffer(value))) + index;
 	ILInt32 result = 0;
 	int ch;
@@ -196,15 +240,19 @@ ILInt32 _IL_Stdio_StdRead_iacii(ILExecThread *thread, ILInt32 fd,
 	}
 	while(count > 0)
 	{
-		ch = ILInputGetChar();
+		ch = getc(stdin);
 		if(ch == -1)
 		{
 			break;
 		}
 		*buf++ = (ILUInt16)(ch & 0xFF);
+		++result;
 		--count;
 	}
 	return result;
+#else
+	return -1;
+#endif
 }
 
 /*
@@ -214,12 +262,16 @@ ILInt32 _IL_Stdio_StdRead_iaBii(ILExecThread *thread, ILInt32 fd,
 								System_Array *value, ILInt32 index,
 								ILInt32 count)
 {
+#ifndef REDUCED_STDIO
 	if(fd != 0)
 	{
 		return -1;
 	}
-	return (ILInt32)ILInputRead
-		(((ILUInt8 *)ArrayToBuffer(value)) + index, (int)count);
+	return (ILInt32)(int)fread
+		(((ILUInt8 *)ArrayToBuffer(value)) + index, 1, (int)count, stdin);
+#else
+	return -1;
+#endif
 }
 
 /*
@@ -227,21 +279,121 @@ ILInt32 _IL_Stdio_StdRead_iaBii(ILExecThread *thread, ILInt32 fd,
  */
 ILInt32 _IL_Stdio_StdPeek(ILExecThread *thread, ILInt32 fd)
 {
+#ifndef REDUCED_STDIO
 	if(fd == 0)
 	{
-		int ch = ILInputGetChar();
+		int ch = getc(stdin);
 		if(ch != -1)
 		{
-			ILInputUngetChar(ch);
+			ungetc(ch, stdin);
 		}
 		return (ILInt32)ch;
 	}
 	else
+#endif
 	{
 		return -1;
 	}
 }
 
+/*
+ * public static void SetConsoleMode(int mode);
+ */
+void _IL_Stdio_SetConsoleMode(ILExecThread *_thread, ILInt32 mode)
+{
+	ILConsoleSetMode(mode);
+}
+
+/*
+ * public static void Beep();
+ */
+void _IL_Stdio_Beep(ILExecThread *_thread)
+{
+	ILConsoleBeep();
+}
+
+/*
+ * public static void Clear();
+ */
+void _IL_Stdio_Clear(ILExecThread *_thread)
+{
+	ILConsoleClear();
+}
+
+/*
+ * public static void ReadKey(out char ch, out int key, out int modifiers);
+ */
+void _IL_Stdio_ReadKey(ILExecThread *_thread, ILUInt16 *ch,
+					   ILInt32 *key, ILInt32 *modifiers)
+{
+	ILConsoleReadKey(ch, key, modifiers);
+}
+
+/*
+ * public static void SetCursorPosition(int x, int y);
+ */
+void _IL_Stdio_SetCursorPosition(ILExecThread *_thread, ILInt32 x, ILInt32 y)
+{
+	ILConsoleSetPosition(x, y);
+}
+
+/*
+ * public static void SetTextAttributes(int attrs);
+ */
+void _IL_Stdio_SetTextAttributes(ILExecThread *_thread, ILInt32 attrs)
+{
+	ILConsoleSetAttributes(attrs);
+}
+
+/*
+ * public static void GetBufferSize(out int width, out int height);
+ */
+void _IL_Stdio_GetBufferSize(ILExecThread *_thread, ILInt32 *width,
+							 ILInt32 *height)
+{
+	ILConsoleGetBufferSize(width, height);
+}
+
+/*
+ * public static void GetCursorPosition(out int x, out int y);
+ */
+void _IL_Stdio_GetCursorPosition(ILExecThread *_thread, ILInt32 *x,
+								 ILInt32 *y)
+{
+	ILConsoleGetPosition(x, y);
+}
+
+/*
+ * public static bool KeyAvailable();
+ */
+ILBool _IL_Stdio_KeyAvailable(ILExecThread *_thread)
+{
+	return ILConsoleKeyAvailable();
+}
+
+/*
+ * public static void SetConsoleTitle(String title);
+ */
+void _IL_Stdio_SetConsoleTitle(ILExecThread *_thread, ILString *title)
+{
+	char *str = ILStringToAnsi(_thread, title);
+	if(str)
+	{
+		ILConsoleSetTitle(str);
+	}
+}
+
+/*
+ * public static void GetWindowSize(out int left, out int top,
+ *									out int width, out int height);
+ */
+void _IL_Stdio_GetWindowSize(ILExecThread *_thread,
+							 ILInt32 *left, ILInt32 *top,
+							 ILInt32 *width, ILInt32 *height)
+{
+	ILConsoleGetWindowSize(left, top, width, height);
+}
+
 #ifdef	__cplusplus
 };
 #endif
