@@ -28,22 +28,33 @@
 extern	"C" {
 #endif
 
-void ILExecInit(unsigned long maxSize)
+int ILExecInit(unsigned long maxSize)
 {
+	/* Create the global trace mutex */
+	if ((globalTraceMutex = ILMutexCreate()) == 0)
+	{
+		return IL_EXEC_INIT_OUTOFMEMORY;
+	}
+
 	/* Initialize the thread routines */	
 	ILThreadInit();
 
 	/* Initialize the global garbage collector */	
-	ILGCInit(maxSize);	
+	ILGCInit(maxSize);
+
+	return IL_EXEC_INIT_OK;
 }
 
 void ILExecDeinit()
-{
+{	
 	/* Deinitialize the global garbage collector */	
 	ILGCDeinit();	
 
 	/* Deinitialize the thread routines */	
 	ILThreadDeinit();	
+
+	/* Destroy the global trace mutex */
+	ILMutexDestroy(globalTraceMutex);
 }
 
 ILExecProcess *ILExecProcessCreate(unsigned long stackSize, unsigned long cachePageSize)
@@ -61,9 +72,6 @@ ILExecProcess *ILExecProcessCreate(unsigned long stackSize, unsigned long cacheP
 	process->firstThread = 0;
 	process->mainThread = 0;
 	process->finalizerThread = 0;
-#ifdef USE_HASHING_MONITORS
-	process->monitorHash = 0;
-#endif
 	process->stackSize = ((stackSize < IL_CONFIG_STACK_SIZE)
 							? IL_CONFIG_STACK_SIZE : stackSize);
 	process->frameStackSize = IL_CONFIG_FRAME_STACK_SIZE;
@@ -125,19 +133,15 @@ ILExecProcess *ILExecProcessCreate(unsigned long stackSize, unsigned long cacheP
 		return 0;
 	}
 
-#ifdef USE_HASHING_MONITORS
-	/* Initialize the monitor system lock */
-	process->monitorSystemLock = ILMutexCreate();
-	if(!(process->monitorSystemLock))
+	/* Initialize the finalization context */
+	process->finalizationContext = (ILFinalizationContext *)ILGCAlloc(sizeof(ILFinalizationContext));
+	if (!process->finalizationContext)
 	{
 		ILExecProcessDestroy(process);
 		return 0;
 	}
-#endif
 
-	/* Initialize the finalization context */
-	process->finalizationContext = (ILFinalizationContext *)ILGCAlloc(sizeof(ILFinalizationContext));
-	if (!process->finalizationContext)
+	if (!_ILExecMonitorProcessCreate(process))
 	{
 		ILExecProcessDestroy(process);
 		return 0;
@@ -187,18 +191,71 @@ struct _tagThreadListEntry
 	ThreadListEntry *next;
 };
 
+/*
+ * Unloads a process and all threads associated with it.
+ * The process may not unload immediately (or even ever).
+ * If the current thread exists inside the process, a
+ * new thread will be created to unload & destroy the
+ * process.  When this function exits, the thread that
+ * calls this method will still be able to execute managed
+ * code even if it resides within the process it tried to
+ * destroy.  The process will eventually be destroyed
+ * when the thread (and all other process-owned threads)
+ * exit.
+ */
+void ILExecProcessUnload(ILExecProcess *process)
+{
+	/* TODO: Implement same semantics as AppDomain.Unload
+	   for embedders */
+}
+
+/*
+ * Destroy a process.
+ *
+ * DO NOT call this function from a thread that expects to be
+ * alive after the function returns.  This is *NOT* an implementation
+ * of AppDomain.Unload.  A thread that calls this function but exists
+ * inside the process will be *unusable* from managed code when this
+ * function exits.
+ *
+ * In the implementation of AppDomain.Unload, this method should
+ * not be called by a thread that runs inside the domain it is trying
+ * to destroy otherwise the thread will return dead & to a dead domain.
+ * A new domain-less thread should be created to destroy the domain.
+ * On single threaded systems, this method can be called directly
+ * by AppDomain.Unload unless the current thread is living in the
+ * domain it is trying to unload in which case AppDomain.Unload *MUST*
+ * just return without doing anything.
+ *
+ * Thong Nguyen (tum@veridicus.com)
+ */
 void ILExecProcessDestroy(ILExecProcess *process)
 {
 	int result;
 	int mainIsFinalizer = 0;
-	ILThread *mainSupportThread;
 	ILExecThread *thread, *nextThread;	
 	ThreadListEntry *firstEntry, *entry, *next;
 
+	/* Determine if this is a single-threaded system where the main
+	   thread is the finalizer thread */
+	mainIsFinalizer = process->mainThread == process->finalizerThread;
+
 	if (process->mainThread)
 	{
-		/* TODO: Stack still contains last frame */
-		ILMemZero(process->mainThread->stackBase, sizeof(int));
+		if (mainIsFinalizer)
+		{
+			/* If the main thread is the finalizer thread then
+			   we have to zero the memory of the CVM stack so that
+			   stray pointers are erased. */
+			ILMemZero(process->mainThread->stackBase, process->stackSize);
+			ILMemZero(process->mainThread->frameStack, process->frameStackSize);
+		}
+		else
+		{
+			/* If the main thread isn't the finalizer then it's
+			   possible to simply destroy the main thread before
+			   calling the finalizers */
+		}
 	}
 
 	/* Delete all managed threads so the objects they used can be finalizerd */
@@ -224,7 +281,7 @@ void ILExecProcessDestroy(ILExecProcess *process)
 				/*
 				 * Instead of aborting and then waiting on each thread, we abort all
 				 * threads and then wait for them all at the bottom.  This prevents us from
-				 *	deadlocking on a thread that's waiting on another (not yet aborted) thread.
+				 * deadlocking on a thread that's waiting on another (not yet aborted) thread.
 				 */
 
 				nextThread = thread->nextThread;
@@ -281,9 +338,8 @@ void ILExecProcessDestroy(ILExecProcess *process)
 			while (entry)
 			{
 				next = entry->next;
-
+				
 				result = ILThreadJoin(entry->thread, -1);
-
 				ILFree(entry);
 
 				if (result != 0)
@@ -312,10 +368,16 @@ void ILExecProcessDestroy(ILExecProcess *process)
 			break;
 		}
 	}
-	
-	mainIsFinalizer = process->mainThread == process->finalizerThread;
-	mainSupportThread = process->mainThread->supportThread;
 
+	/* Unregister (and destroy) the current thread if it isn't needed
+	   for finalization and if it belongs to this domain. */
+	if (!mainIsFinalizer 
+		&& ILExecThreadCurrent() != process->finalizerThread
+		&& ILExecThreadCurrent()->process == process)
+	{
+		ILThreadUnregisterForManagedExecution(ILThreadSelf());
+	}	
+	
 	/* Invoke the finalizers -- hopefully finalizes all objects left in the process being destroyed */
 	/* Any objects left lingering (because of a stray or fake pointer) are orphans */
 	ILGCCollect();
@@ -335,19 +397,15 @@ void ILExecProcessDestroy(ILExecProcess *process)
 	/* Reenable finalizers */
 	ILGCEnableFinalizers();	
 
-	/* Abort the main thread if we aren't the main thread or the finalizer thread*/
-	if (!mainIsFinalizer && ILThreadSelf() != mainSupportThread)
+	/* Unregister (and destroy) the current thread if it 
+	   wasn't destroyed above and if it belongs to this domain */
+	if (mainIsFinalizer
+		&& ILExecThreadCurrent() != process->finalizerThread
+		&& ILExecThreadCurrent()->process == process)
 	{
-		/* Abort the thread */
-		ILThreadAbort(mainSupportThread);		
+		ILThreadUnregisterForManagedExecution(ILThreadSelf());
 	}
 
-	/* Unregister the main thread since its engine thread is now gone */
-	if (ILThreadSelf() == mainSupportThread)
-	{
-		ILThreadUnregisterForManagedExecution(mainSupportThread);
-	}
-	
 	/* Destroy the finalizer thread */
 	if (process->finalizerThread)
 	{
@@ -441,10 +499,7 @@ void ILExecProcessDestroy(ILExecProcess *process)
 		ILMemZero(process->randomPool, sizeof(process->randomPool));
 	}
 
-#ifdef USE_HASHING_MONITORS
-	/* Destroy the monitor system lock */
-	ILMutexDestroy(process->monitorSystemLock);
-#endif
+	_ILExecMonitorProcessDestroy(process);
 
 	if (process->lock)
 	{
@@ -455,6 +510,9 @@ void ILExecProcessDestroy(ILExecProcess *process)
 	/* Free the process block itself */
 	ILGCFreePersistent(process);
 
+	ILGCCollect();
+	ILGCInvokeFinalizers();
+
 	/* Reset the console to the "normal" mode */
 	ILConsoleSetMode(IL_CONSOLE_NORMAL);
 }
@@ -588,6 +646,10 @@ ILMethod *ILExecProcessGetEntry(ILExecProcess *process)
 	ILRWLockReadLock(process->metadataLock);
 	while((image = ILContextNextImage(process->context, image)) != 0)
 	{
+		if(ILImageType(image) != IL_IMAGETYPE_EXE)
+		{
+			continue;
+		}
 		token = ILImageGetEntryPoint(image);
 		if(token && (token & IL_META_TOKEN_MASK) == IL_META_TOKEN_METHOD_DEF)
 		{
