@@ -21,59 +21,85 @@
  */
 
 #include "engine_private.h"
+#include "lib_defs.h"
 
 #ifdef	__cplusplus
 extern	"C" {
 #endif
 
+/*
+ * Gets the currently ILExecThread.
+ */
 ILExecThread *ILExecThreadCurrent(void)
 {
-	return (ILExecThread *)(ILThreadGetObject(ILThreadSelf()));
+	return ILExecThreadFromThread(ILThreadSelf());
 }
 
 /*
- *	Gets the current CLR thread.
+ * Get an ILExecThread from the given thread.
  */
-ILObject *_ILGetCurrentClrThread(ILExecThread *thread)
+ILExecThread *ILExecThreadFromThread(ILThread *thread)
+{
+	return (ILExecThread *)(ILThreadGetObject(thread));
+}
+
+/*
+ * Gets the managed thread object from an engine thread.
+ */
+ILObject *ILExecThreadGetClrThread(ILExecThread *thread)
 {
 	ILClass *classInfo;
-	ILObject *clrThread;
+	System_Thread *clrThread;
 
-	if (thread->clrThread == NULL)
+	if (thread->clrThread == 0)
 	{
-		/*
-		* Main thread or another thread created from inside the engine.
-		*/
-
+		/* Main thread or another thread created from inside the engine. */
+	
 		/* Get the CLR thread class */
 
 		classInfo = ILExecThreadLookupClass(thread, "System.Threading.Thread");		
 
 		/* Allocate a new CLR thread object */
 
-		clrThread = _ILEngineAllocObject(thread, classInfo);
+		clrThread = (System_Thread *)_ILEngineAllocObject(thread, classInfo);
 
 		/* Associate the CLR thread object with the OS thread */
 
-		*((ILThread **)clrThread) = ILThreadSelf();
+		clrThread->privateData = ILThreadSelf();
 
 		/* Associate the executing thread with the CLR thread */
-		thread->clrThread = clrThread;
+		thread->clrThread = (ILObject *)clrThread;
+	}
 
-		return clrThread;
-	}
-	else
-	{
-		return thread->clrThread;
-	}
+	return thread->clrThread;
 }
 
 /*
- *	Makes the given support thread execute in the context of the given engine thread.
+ * Associates a support thread with an engine thread and the engine
+ * thread with the support thread.
  */
 void _ILThreadExecuteOn(ILThread *thread, ILExecThread *execThread)
 {
 	ILThreadSetObject(thread, execThread);
+	execThread->supportThread = thread;
+}
+
+/*
+ * Disassociates a support thread with an engine thread and the
+ * engine thread with the support thread.
+ * The support thread will no longer be able to execute managed
+ * code and the engine thread will no longer be able to be used
+ * to execute anything until it is reassociated using ILThreadExecuteOn.
+ */
+void _ILThreadUnexecuteOn(ILThread *thread, ILExecThread *execThread)
+{
+	if (ILThreadGetObject(thread) != execThread)
+	{
+		return;
+	}
+
+	ILThreadSetObject(thread, 0);
+	execThread->supportThread = 0;
 }
 
 /*
@@ -103,11 +129,10 @@ ILExecThread *ILThreadRegisterForManagedExecution(ILExecProcess *process, ILThre
 		return 0;
 	}
 
-	/* Associate the new engine-level thread with the OS-level thread */
-	_ILThreadExecuteOn(thread, execThread);
+	/* TODO: Notify the GC that we possibly have a new thread to be scanned */
 
-	/* Associate the OS-level thread with the new engine-level thread */
-	execThread->supportThread = thread;
+	/* Associate the new engine-level thread with the OS-level thread (and vice-versa) */
+	_ILThreadExecuteOn(thread, execThread);
 
 	/* Register a cleanup handler for the thread */
 	ILThreadRegisterCleanup(thread, ILExecThreadCleanup);
@@ -135,7 +160,7 @@ void ILThreadUnregisterForManagedExecution(ILThread *thread)
 	ILThreadUnregisterCleanup(thread, ILExecThreadCleanup);
 
 	/* Disassociate the engine thread with the support thread */
-	_ILThreadExecuteOn(thread, 0);
+	_ILThreadUnexecuteOn(thread, execThread);
 
 	/* Destroy the engine thread */
 	_ILExecThreadDestroy(execThread);
@@ -175,9 +200,10 @@ ILExecThread *_ILExecThreadCreate(ILExecProcess *process)
 
 	/* Initialize the thread state */
 	thread->supportThread = 0;
+	thread->clrThread = 0;
 	thread->aborting = 0;
-	thread->clrThread = 0;	
-	thread->freeMonitor = 0;	
+	thread->freeMonitor = 0;
+	thread->freeMonitorCount = 0;
 	thread->pc = 0;
 	thread->isFinalizerThread = 0;
 	thread->frame = thread->stackBase;
@@ -186,7 +212,13 @@ ILExecThread *_ILExecThreadCreate(ILExecProcess *process)
 	thread->thrownException = 0;	
 	thread->threadStaticSlots = 0;
 	thread->threadStaticSlotsUsed = 0;
-
+	thread->currentException = 0;
+	thread->abortRequested = 0;	
+	thread->runningManagedCode = 0;	
+	thread->abortHandlerEndPC = 0;
+	thread->threadAbortException = 0;
+	thread->abortHandlerFrame = 0;
+	
 	/* Attach the thread to the process */
 	ILMutexLock(process->lock);
 	thread->process = process;
@@ -205,8 +237,7 @@ ILExecThread *_ILExecThreadCreate(ILExecProcess *process)
 }
 
 void _ILExecThreadDestroy(ILExecThread *thread)
-{
-	ILExecMonitor *monitor, *next;
+{	
 	ILExecProcess *process = thread->process;
 
 	/* Lock down the process */
@@ -223,17 +254,6 @@ void _ILExecThreadDestroy(ILExecThread *thread)
 		process->finalizerThread = 0;
 	}
 
-	/* Delete the free monitors list */
-
-	monitor = thread->freeMonitor;
-
-	while (monitor)
-	{
-		next = monitor->next;
-		ILExecMonitorDestroy(monitor);
-		monitor = next;
-	}
-
 	/* Detach the thread from its process */
 	if(thread->nextThread)
 	{
@@ -248,6 +268,13 @@ void _ILExecThreadDestroy(ILExecThread *thread)
 		process->firstThread = thread->nextThread;
 	}
 
+	/* Remove associations between ILExecThread and ILThread if they
+	   haven't already been removed */
+	if (thread->supportThread)
+	{		
+		_ILThreadUnexecuteOn(thread->supportThread, thread);
+	}
+
 	/* Destroy the operand stack */
 	ILGCFreePersistent(thread->stackBase);
 
