@@ -1068,12 +1068,9 @@ static int processImage(ILLinker *linker, const char *filename,
 						int useStdlib, int isFirstFile)
 {
 	int errors = 0;
-	int model, alignFlags;
-	char libcName[64];
 
 	/* Add the image to the linker context */
-	model = ILLinkerCMemoryModel(image, &alignFlags);
-	if(model != 0)
+	if(ILLinkerIsCObject(image))
 	{
 		/* Load the standard C libraries that we need for linking */
 		if(isFirstFile)
@@ -1093,16 +1090,14 @@ static int processImage(ILLinker *linker, const char *filename,
 			{
 				if(!stdCLibrary)
 				{
-					sprintf(libcName, "libc%d", model);
-					stdCLibrary = libcName;
+					stdCLibrary = "libc";
 				}
 				errors |= addLibrary(linker, stdCLibrary);
 			}
 		}
 
 		/* Add the C object file to the linker */
-		if(!ILLinkerAddCObject(linker, context, image,
-							   filename, model, alignFlags))
+		if(!ILLinkerAddCObject(linker, context, image, filename))
 		{
 			errors |= 1;
 		}
