@@ -114,6 +114,18 @@ static int ClassHash_Match(const ILClassName *classInfo,
 	}
 
 	/* Match the scope */
+	if(key->scopeItem &&
+	   (key->scopeItem->token & IL_META_TOKEN_MASK) == IL_META_TOKEN_MODULE)
+	{
+		/* Exported types in a file scope will also match module requests */
+		if(classInfo->scope &&
+	       (classInfo->scope->token & IL_META_TOKEN_MASK) ==
+		   			IL_META_TOKEN_FILE &&
+		   key->scopeItem->image == classInfo->scope->image)
+		{
+			goto moduleScope;
+		}
+	}
 	if(key->scopeItem && key->scopeItem != classInfo->scope)
 	{
 		return 0;
@@ -122,6 +134,7 @@ static int ClassHash_Match(const ILClassName *classInfo,
 	{
 		return 0;
 	}
+moduleScope:
 
 	/* Match the image */
 	if(key->image && key->image != classInfo->image)
@@ -265,6 +278,12 @@ void ILContextDestroy(ILContext *context)
 	/* Destroy the type pool */
 	ILMemPoolDestroy(&(context->typePool));
 
+	/* Destory the redo table */
+	if(context->redoItems)
+	{
+		ILFree(context->redoItems);
+	}
+
 	/* Destroy the context itself */
 	ILFree(context);
 }
