@@ -22,6 +22,8 @@
 #include "program.h"
 #include "il_jopcodes.h"
 
+#ifdef IL_CONFIG_JAVA
+
 #ifdef	__cplusplus
 extern	"C" {
 #endif
@@ -1167,3 +1169,5 @@ cleanup:
 #ifdef	__cplusplus
 };
 #endif
+
+#endif /* IL_CONFIG_JAVA */
