@@ -20,6 +20,8 @@
 
 #include "program.h"
 
+#ifdef IL_USE_WRITER
+
 #ifdef	__cplusplus
 extern	"C" {
 #endif
@@ -121,10 +123,10 @@ static void Format_TypeRef(ILWriter *writer, ILImage *image,
 			values[IL_OFFSET_TYPEREF_SCOPE] = 0;
 		}
 	}
-	else if(info->scope)
+	else if(info->className->scope)
 	{
 		/* Refers to a type in a foreign scope */
-		values[IL_OFFSET_TYPEREF_SCOPE] = info->scope->token;
+		values[IL_OFFSET_TYPEREF_SCOPE] = info->className->scope->token;
 	}
 	else if(!ILClassIsRef(info))
 	{
@@ -136,9 +138,42 @@ static void Format_TypeRef(ILWriter *writer, ILImage *image,
 		/* Unknown scope */
 		values[IL_OFFSET_TYPEREF_SCOPE] = 0;
 	}
-	values[IL_OFFSET_TYPEREF_NAME] = GetPersistString(image, info->name);
+	values[IL_OFFSET_TYPEREF_NAME] =
+			GetPersistString(image, info->className->name);
 	values[IL_OFFSET_TYPEREF_NAMESPACE] =
-			GetPersistString(image, info->namespace);
+			GetPersistString(image, info->className->namespace);
+}
+
+/*
+ * Search the TypeSpec table for a particular type.
+ */
+static ILToken TypeToToken(ILImage *image, ILType *type)
+{
+	ILTypeSpec *spec = 0;
+	while((spec = (ILTypeSpec *)ILImageNextToken
+				(image, IL_META_TOKEN_TYPE_SPEC, spec)) != 0)
+	{
+		if(ILTypeIdentical(ILTypeSpec_Type(spec), type))
+		{
+			return ILTypeSpec_Token(spec);
+		}
+	}
+	return 0;
+}
+
+/*
+ * Convert an ILClass into a TypeRef, TypeDef, or TypeSpec token.
+ */
+static ILToken ClassToToken(ILImage *image, ILClass *info)
+{
+	if(!(info->synthetic))
+	{
+		return info->programItem.token;
+	}
+	else
+	{
+		return TypeToToken(image, info->synthetic);
+	}
 }
 
 /*
@@ -155,12 +190,13 @@ static void Format_TypeDef(ILWriter *writer, ILImage *image,
 	/* Set the easy fields */
 	values[IL_OFFSET_TYPEDEF_ATTRS] =
 		(info->attributes & ~IL_META_TYPEDEF_SYSTEM_MASK);
-	values[IL_OFFSET_TYPEDEF_NAME] = GetPersistString(image, info->name);
+	values[IL_OFFSET_TYPEDEF_NAME] =
+			GetPersistString(image, info->className->name);
 	values[IL_OFFSET_TYPEDEF_NAMESPACE] =
-			GetPersistString(image, info->namespace);
+			GetPersistString(image, info->className->namespace);
 	if(info->parent)
 	{
-		values[IL_OFFSET_TYPEDEF_PARENT] = info->parent->programItem.token;
+		values[IL_OFFSET_TYPEDEF_PARENT] = ClassToToken(image, info->parent);
 	}
 	else
 	{
@@ -293,7 +329,8 @@ static void Format_InterfaceImpl(ILWriter *writer, ILImage *image,
 						 		 ILUInt32 *values, ILImplements *impl)
 {
 	values[IL_OFFSET_INTERFACE_TYPE] = impl->implement->programItem.token;
-	values[IL_OFFSET_INTERFACE_INTERFACE] = impl->interface->programItem.token;
+	values[IL_OFFSET_INTERFACE_INTERFACE] =
+			ClassToToken(image, impl->interface);
 }
 
 /*
@@ -309,12 +346,12 @@ static void Format_MemberRef(ILWriter *writer, ILImage *image,
 	{
 		ref = (ILMemberRef *)member;
 		values[IL_OFFSET_MEMBERREF_PARENT] =
-				ref->ref->owner->programItem.token;
+				ClassToToken(image, ref->ref->owner);
 	}
 	else
 	{
 		values[IL_OFFSET_MEMBERREF_PARENT] =
-				member->owner->programItem.token;
+				ClassToToken(image, member->owner);
 	}
 }
 
@@ -424,7 +461,8 @@ static void Format_Event(ILWriter *writer, ILImage *image,
 	}
 	else
 	{
-		values[IL_OFFSET_EVENT_TYPE] = 0;
+		values[IL_OFFSET_EVENT_TYPE] =
+				TypeToToken(image, event->member.signature);
 	}
 }
 
@@ -501,7 +539,7 @@ static void Format_ImplMap(ILWriter *writer, ILImage *image,
 {
 	values[IL_OFFSET_IMPLMAP_ATTRS] = pinvoke->member.attributes;
 	values[IL_OFFSET_IMPLMAP_METHOD] =
-			pinvoke->method->member.programItem.token;
+			pinvoke->memberInfo->programItem.token;
 	values[IL_OFFSET_IMPLMAP_ALIAS] =
 			GetPersistString(image, pinvoke->aliasName);
 	values[IL_OFFSET_IMPLMAP_MODULE] = pinvoke->module->programItem.token;
@@ -613,12 +651,21 @@ static void Format_File(ILWriter *writer, ILImage *image,
 static void Format_ExportedType(ILWriter *writer, ILImage *image,
 	 		            		ILUInt32 *values, ILExportedType *type)
 {
-	values[IL_OFFSET_EXPTYPE_ATTRS] = type->attributes;
+	values[IL_OFFSET_EXPTYPE_ATTRS] = type->classItem.attributes;
 	values[IL_OFFSET_EXPTYPE_CLASS] = type->identifier;
-	values[IL_OFFSET_EXPTYPE_NAME] = GetPersistString(image, type->name);
+	values[IL_OFFSET_EXPTYPE_NAME] =
+			GetPersistString(image, type->classItem.className->name);
 	values[IL_OFFSET_EXPTYPE_NAMESPACE] =
-			GetPersistString(image, type->namespace);
-	values[IL_OFFSET_EXPTYPE_FILE] = type->scope->token;
+			GetPersistString(image, type->classItem.className->namespace);
+	if(type->classItem.className->scope)
+	{
+		values[IL_OFFSET_EXPTYPE_FILE] =
+			type->classItem.className->scope->token;
+	}
+	else
+	{
+		values[IL_OFFSET_EXPTYPE_FILE] = 0;
+	}
 }
 
 /*
@@ -644,6 +691,32 @@ static void Format_NestedClass(ILWriter *writer, ILImage *image,
 	values[IL_OFFSET_NESTEDCLASS_PARENT] = nested->parent->programItem.token;
 }
 
+/*
+ * Format a GenericPar token.
+ */
+static void Format_GenericPar(ILWriter *writer, ILImage *image,
+	 		                  ILUInt32 *values, ILGenericPar *genPar)
+{
+	values[IL_OFFSET_GENERICPAR_NUMBER] = genPar->number;
+	values[IL_OFFSET_GENERICPAR_FLAGS] = genPar->flags;
+	values[IL_OFFSET_GENERICPAR_OWNER] = genPar->ownedItem.owner->token;
+	values[IL_OFFSET_GENERICPAR_NAME] = GetPersistString(image, genPar->name);
+	values[IL_OFFSET_GENERICPAR_KIND] =
+			(genPar->kind ? genPar->kind->token : 0);
+	values[IL_OFFSET_GENERICPAR_CONSTRAINT] =
+			(genPar->constraint ? genPar->constraint->token : 0);
+}
+
+/*
+ * Format a MethodSpec token.
+ */
+static void Format_MethodSpec(ILWriter *writer, ILImage *image,
+	 		                  ILUInt32 *values, ILMethodSpec *spec)
+{
+	values[IL_OFFSET_METHODSPEC_METHOD] = spec->method->programItem.token;
+	values[IL_OFFSET_METHODSPEC_INST_RAW] = spec->typeBlob;
+}
+
 /*
  * Array of all formatting routines for the known token types.
  */
@@ -692,8 +765,8 @@ static ILFormatFunc const Formatters[64] = {
 	(ILFormatFunc)Format_ExportedType,
 	(ILFormatFunc)Format_ManifestResource,		/* 28 */
 	(ILFormatFunc)Format_NestedClass,
-	0,
-	0,
+	(ILFormatFunc)Format_GenericPar,
+	(ILFormatFunc)Format_MethodSpec,
 	0,
 	0,
 	0,
@@ -855,6 +928,23 @@ static int SigWrite_TypeSpec(ILTypeSpec *spec)
 	return 1;
 }
 
+/*
+ * Write signature information for a method specification.
+ */
+static int SigWrite_MethodSpec(ILMethodSpec *spec)
+{
+	if(spec->type && !(spec->typeBlob))
+	{
+		spec->typeBlob =
+			ILTypeToMethodSig(spec->programItem.image, spec->type);
+		if(!(spec->typeBlob))
+		{
+			return 0;
+		}
+	}
+	return 1;
+}
+
 /*
  * Array of all signature writing routines for the known token types.
  */
@@ -903,7 +993,7 @@ static ILSigWriteFunc const SigWriters[64] = {
 	0,											/* 28 */
 	0,
 	0,
-	0,
+	(ILSigWriteFunc)SigWrite_MethodSpec,
 	0,
 	0,
 	0,
@@ -1143,9 +1233,24 @@ static int Sort_MethodImpl(ILOverride **over1, ILOverride **over2)
  */
 static int Sort_ImplMap(ILPInvoke **pinvoke1, ILPInvoke **pinvoke2)
 {
-	ILToken token1 = (*pinvoke1)->method->member.programItem.token;
-	ILToken token2 = (*pinvoke2)->method->member.programItem.token;
-	if(token1 < token2)
+	ILToken token1 = (*pinvoke1)->memberInfo->programItem.token;
+	ILToken token2 = (*pinvoke2)->memberInfo->programItem.token;
+	ILToken tokenNum1 = (token1 & ~IL_META_TOKEN_MASK);
+	ILToken tokenNum2 = (token2 & ~IL_META_TOKEN_MASK);
+
+	/* Compare the bottom parts of the token first, because
+	   the table must be sorted on its encoded value, not
+	   on the original value.  Encoded values put the token
+	   type in the low order bits */
+	if(tokenNum1 < tokenNum2)
+	{
+		return -1;
+	}
+	else if(tokenNum1 > tokenNum2)
+	{
+		return 1;
+	}
+	else if(token1 < token2)
 	{
 		return -1;
 	}
@@ -1227,7 +1332,7 @@ static ILSortFunc const SortFuncs[64] = {
 	0,
 	0,											/* 28 */
 	(ILSortFunc)Sort_NestedClass,
-	0,
+	(ILSortFunc)Sort_OwnedItem,
 	0,
 	0,
 	0,
@@ -1517,3 +1622,5 @@ int _ILWriteMetadataIndex(ILWriter *writer, ILImage *image)
 #ifdef	__cplusplus
 };
 #endif
+
+#endif /* IL_USE_WRITER */
