@@ -175,12 +175,17 @@ typedef int (*ILExecDebugHookFunc)(void *userData,
 #define	IL_HOOK_CONTINUE			0
 #define	IL_HOOK_ABORT				1
 
+#define IL_EXEC_INIT_OK				(0)
+#define IL_EXEC_INIT_OUTOFMEMORY	(1)
+
 /*
  * Initialize the engine and set a default maximum heap size.
  * If the size is zero, then use all of memory for the heap.
  * This should be called only once per application.
+ *
+ * Returns 0 if the engine was successfully initialized.
  */
-void ILExecInit(unsigned long maxSize);
+int ILExecInit(unsigned long maxSize);
 
 /*
  *	Deinitialize the engine.
@@ -204,9 +209,27 @@ ILExecProcess *ILExecProcessCreate(unsigned long frameStackSize, unsigned long c
 
 /*
  * Destroy a process and all threads associated with it.
+ * If the thread that calls this method belongs to the process
+ * the thread will no longer be able to execute managed code
+ * when the function exits.  For a safer version that can
+ * be called from managed code, see ILExecProcessUnload.
  */
 void ILExecProcessDestroy(ILExecProcess *process);
 
+/*
+ * Unloads a process and all threads associated with it.
+ * The process may not unload immediately (or even ever).
+ * If the current thread exists inside the process, a
+ * new thread will be created to unload & destroy the
+ * process.  When this function exits, the thread that
+ * calls this method will still be able to execute managed
+ * code even if it resides within the process it tried to
+ * destroy.  The process will eventually be destroyed
+ * when the thread (and all other process-owned threads)
+ * exit.
+ */
+void ILExecProcessUnload(ILExecProcess *process);
+
 /*
  * Set the list of directories to be used for library path
  * searching, before inspecting the standard directories.
@@ -328,11 +351,20 @@ int ILExecProcessAddInternalCallTable(ILExecProcess* process,
 					const ILEngineInternalClassInfo* internalClassTable,
 					int internalClassCount);
 /*
- * Get the current thread from a PInvoke'd method.  The behaviour
- * is undefined if used anywhere else.
+ * Get the current ILExecThread.
  */
 ILExecThread *ILExecThreadCurrent(void);
 
+/*
+ * Get an ILExecThread from the given thread.
+ */
+ILExecThread *ILExecThreadFromThread(ILThread *thread);
+
+/*
+ *	Gets the managed thread object from an engine thread.
+ */
+ILObject *ILExecThreadGetClrThread(ILExecThread *thread);
+
 /*
  * Get the process that corresponds to a thread.
  */
@@ -507,6 +539,11 @@ void ILExecThreadSetException(ILExecThread *thread, ILObject *obj);
  */
 void ILExecThreadClearException(ILExecThread *thread);
 
+/*
+ * Throws the given exception.
+ */
+void ILExecThreadThrow(ILExecThread *thread, ILObject *obj);
+
 /*
  * Throw a system exception with a particular type and
  * resource name.
@@ -656,6 +693,11 @@ int ILExecThreadGetElem(ILExecThread *thread, void *value,
 int ILExecThreadSetElem(ILExecThread *thread, ILObject *array,
 						ILInt32 index, ...);
 
+/*
+ * Create a box for a type and initialize the box with no value.
+ */
+ILObject *ILExecThreadBoxNoValue(ILExecThread *thread, ILType *type);
+
 /*
  * Box the value at a pointer using a specific primitive or value type.
  */
@@ -696,6 +738,11 @@ int ILExecThreadPromoteAndUnbox(ILExecThread *thread, ILType *type,
  */
 void ILExecThreadPrintException(ILExecThread *thread);
 
+/*
+ * Checks if the given object is a ThreadAbortException.
+ */
+int ILExecThreadIsThreadAbortException(ILExecThread *thread, ILObject *object);
+
 #ifdef	__cplusplus
 };
 #endif
