@@ -65,6 +65,8 @@ extern	"C" {
 #define	IL_META_TOKEN_EXPORTED_TYPE				0x27000000
 #define	IL_META_TOKEN_MANIFEST_RESOURCE			0x28000000
 #define	IL_META_TOKEN_NESTED_CLASS				0x29000000
+#define	IL_META_TOKEN_GENERIC_PAR				0x2A000000
+#define	IL_META_TOKEN_METHOD_SPEC				0x2B000000
 #define	IL_META_TOKEN_STRING					0x70000000
 #define	IL_META_TOKEN_NAME						0x71000000
 #define	IL_META_TOKEN_BASE_TYPE					0x72000000
@@ -320,9 +322,9 @@ extern	"C" {
 #define	IL_META_ELEMTYPE_VALUETYPE				0x11
 #define	IL_META_ELEMTYPE_VALUECLASS				IL_META_ELEMTYPE_VALUETYPE
 #define	IL_META_ELEMTYPE_CLASS					0x12
-#define	IL_META_ELEMTYPE_UNUSED1				0x13	/* !! */
+#define	IL_META_ELEMTYPE_VAR					0x13
 #define	IL_META_ELEMTYPE_ARRAY					0x14
-#define	IL_META_ELEMTYPE_COPYCTOR				0x15
+#define	IL_META_ELEMTYPE_WITH					0x15
 #define	IL_META_ELEMTYPE_TYPEDBYREF				0x16
 #define	IL_META_ELEMTYPE_VALUEARRAY				0x17
 #define	IL_META_ELEMTYPE_I						0x18
@@ -331,11 +333,11 @@ extern	"C" {
 #define	IL_META_ELEMTYPE_FNPTR					0x1B
 #define	IL_META_ELEMTYPE_OBJECT					0x1C
 #define	IL_META_ELEMTYPE_SZARRAY				0x1D
-#define	IL_META_ELEMTYPE_GENERICARRAY			0x1E
+#define	IL_META_ELEMTYPE_MVAR					0x1E
 #define	IL_META_ELEMTYPE_CMOD_REQD				0x1F
 #define	IL_META_ELEMTYPE_CMOD_OPT				0x20
 #define	IL_META_ELEMTYPE_INTERNAL				0x21
-#define	IL_META_ELEMTYPE_VAR					0x22
+#define	IL_META_ELEMTYPE_VAR_OLD				0x22
 #define	IL_META_ELEMTYPE_NAME					0x23
 #define	IL_META_ELEMTYPE_MODIFIER				0x40
 #define	IL_META_ELEMTYPE_SENTINEL				0x41
@@ -378,6 +380,8 @@ extern	"C" {
 #define	IL_META_CALLCONV_LOCAL_SIG				0x07
 #define	IL_META_CALLCONV_PROPERTY				0x08
 #define	IL_META_CALLCONV_UNMGD					0x09
+#define	IL_META_CALLCONV_INSTANTIATION			0x0A
+#define	IL_META_CALLCONV_GENERIC				0x10
 #define	IL_META_CALLCONV_HASTHIS				0x20
 #define	IL_META_CALLCONV_EXPLICITTHIS			0x40
 
@@ -541,6 +545,10 @@ extern	"C" {
 #define	IL_META_MARSHAL_ARRAY					0x0004
 #define	IL_META_MARSHAL_REF_ANSI_ARRAY			0x0005
 #define	IL_META_MARSHAL_REF_UTF8_ARRAY			0x0006
+#define	IL_META_MARSHAL_UTF16_STRING			0x0007
+#define	IL_META_MARSHAL_CUSTOM					0x0008
+#define	IL_META_MARSHAL_ANSI_ARRAY				0x0009
+#define	IL_META_MARSHAL_UTF8_ARRAY				0x000A
 
 /*
  * Control data structure for reading from the metadata section.
