@@ -135,22 +135,29 @@ void ILThreadResume(ILThread *thread);
 void ILThreadInterrupt(ILThread *thread);
 
 /*
- * Abort a thread.  Returns non-zero if the current thread is
- * being aborted and a previous request hasn't been processed.
+ * Request that a thread be aborted.  Returns zero if the thread
+ * is already aborting or already has an abort request queued.
  */
 int ILThreadAbort(ILThread *thread);
 
+/*
+ * Called by the current thread when it is ready to being the
+ * abort process.  The thread moves from an IL_TS_ABORT_REQUESTED
+ * state to an IL_TS_ABORTED state.
+ */
+int ILThreadSelfAborting();
+
 /*
  * Determine if the current thread is in the process of
  * aborting.  If this function returns non-zero, the current
- * thread should call "ILThreadAbort" on itself at some
+ * thread should call "ILThreadSelfAborting" on at some
  * future point to determine if we are processing an abort
  * request, or are in the middle of an abort.  i.e. the
  * correct way to check for aborts is as follows:
  *
  *		if(ILThreadIsAborting())
  *		{
- *			if(ILThreadAbort(ILThreadSelf()))
+ *			if(ILThreadSelfAborting)
  *			{
  *				// Abort request received: do initial processing.
  *				...
@@ -164,6 +171,11 @@ int ILThreadAbort(ILThread *thread);
  */
 int ILThreadIsAborting(void);
 
+/*
+ * Returns 1 if an abort has been requested.
+ */
+int ILThreadIsAbortRequested(void);
+
 /*
  * Reset a pending abort on the current thread.  Returns
  * zero if an abort is not pending.
@@ -197,7 +209,8 @@ int ILThreadGetPriority(ILThread *thread);
 #define	IL_JOIN_INTERRUPTED	2	/* Join was interrupted */
 #define	IL_JOIN_ABORTED		3	/* Thread doing the join was aborted */
 #define	IL_JOIN_SELF		4	/* Tried to join with ourselves */
-#define	IL_JOIN_MEMORY		5	/* Out of memory */
+#define IL_JOIN_UNSTARTED   5   /* Tried to join a thread that hasn't started */
+#define	IL_JOIN_MEMORY		6	/* Out of memory */
 
 /*
  * Join with another thread to wait for exit.
@@ -247,11 +260,16 @@ void ILThreadGetCounts(unsigned long *numForeground,
  * Put a thread to sleep for a given number of milliseconds.
  * Specifying "ms == 0" is the same as yielding the thread.
  * Specifying "ms == IL_MAX_UINT32" will sleep forever.
- * Returns zero if the sleep was interrupted, -1 if the
- * thread was aborted, and 1 if the sleep completed.
+ * Returns 1 on sucess and IL_WAIT_TIMEOUT, IL_WAIT_ABORTED
+ * or IL_WAIT_INTERRUPTED.
  */
 int ILThreadSleep(ILUInt32 ms);
 
+/*
+ * Yield CPUs time to threads of equal or higher priority.
+ */
+void ILThreadYield();
+
 /*
  *	Wait for a specified amount of time (in ms) for a all foreground threads
  * (except the main thread) to finish.  A timeout of -1 means infinite.
@@ -339,10 +357,10 @@ int ILWaitHandleClose(ILWaitHandle *handle);
 /*
  * Return values for "ILWaitOne", "ILWaitAny", and "ILWaitAll".
  */
-#define	IL_WAIT_TIMEOUT			-1
-#define	IL_WAIT_FAILED			-2
-#define	IL_WAIT_INTERRUPTED		-3
-#define	IL_WAIT_ABORTED			-4
+#define	IL_WAIT_TIMEOUT			(-1)
+#define	IL_WAIT_FAILED			(-2)
+#define	IL_WAIT_INTERRUPTED		(-3)
+#define	IL_WAIT_ABORTED			(-4)
 
 /*
  * Wait for one wait handle to become available.  Returns
@@ -378,8 +396,7 @@ ILWaitHandle *ILWaitMutexCreate(int initiallyOwned);
  * Create a named wait mutex, or return an existing named
  * mutex with the same name.
  */
-ILWaitHandle *ILWaitMutexNamedCreate(const char *name, int initiallyOwned,
-									 int *gotOwnership);
+ILWaitHandle *ILWaitMutexNamedCreate(const char *name, int initiallyOwned, int *gotOwnership);
 
 /*
  * Release a wait mutex that is currently held by the
@@ -409,22 +426,9 @@ int ILWaitMonitorPulse(ILWaitHandle *handle);
  */
 int ILWaitMonitorPulseAll(ILWaitHandle *handle);
 
-/*
- *	Checks if the monitor has no owner and no waiters.
- */
-int ILWaitMonitorCanClose(ILWaitHandle *handle);
-
-#define IL_WAIT_LEAVE_STILL_OWNS		(2)
-
-/*
- *	Similar to ILWaitMonitorLeave except that waiting moniters aren't signalled.
- */
-int ILWaitMonitorSpeculativeLeave(ILWaitHandle *handle);
-
-/*
- *	Signals waiting moniters when call after ILWaitMonitorSpeculativeLeave.
- */
-int ILWaitMonitorCompleteLeave(ILWaitHandle *handle);
+#define IL_WAITMUTEX_RELEASE_SUCCESS	(1)
+#define IL_WAITMUTEX_RELEASE_STILL_OWNS	(2)
+#define IL_WAITMUTEX_RELEASE_FAIL	(0)
 
 /*
  * Syntactic sugar.
@@ -435,7 +439,6 @@ int ILWaitMonitorCompleteLeave(ILWaitHandle *handle);
 			(ILWaitOne((monitor), (timeout)))
 #define	ILWaitMonitorLeave(monitor)		\
 			(ILWaitMutexRelease((monitor)))
-
 /*
  * Wait Event definitions
  *
