@@ -3,6 +3,8 @@
  *
  * Copyright (C) 2002  Southern Storm Software, Pty Ltd.
  *
+ * Contributions:  Thong Nguyen (tum@veridicus.com)
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -30,6 +32,8 @@ from C# code and can be held by a suspended thread.
 */
 
 #include "thr_defs.h"
+#include "wait_mutex.h"
+#include "interlocked.h"
 
 #ifdef	__cplusplus
 extern	"C" {
@@ -56,6 +60,7 @@ static int MutexClose(ILWaitMutex *mutex)
 {
 	/* Lock down the mutex and determine if it is currently owned */
 	_ILMutexLock(&(mutex->parent.lock));
+
 	if(mutex->owner != 0 || !_ILWakeupQueueIsEmpty(&(mutex->queue)))
 	{
 		_ILMutexUnlock(&(mutex->parent.lock));
@@ -66,6 +71,7 @@ static int MutexClose(ILWaitMutex *mutex)
 	_ILMutexUnlock(&(mutex->parent.lock));
 	_ILWakeupQueueDestroy(&(mutex->queue));
 	_ILMutexDestroy(&(mutex->parent.lock));
+	
 	return IL_WAITCLOSE_FREE;
 }
 
@@ -243,7 +249,7 @@ static void MutexNameListInit(void)
 }
 
 ILWaitHandle *ILWaitMutexNamedCreate(const char *name, int initiallyOwned,
-									 int *gotOwnership)
+							 int *gotOwnership)
 {
 	ILWaitHandle *handle;
 	ILWaitMutexNamed *mutex;
@@ -349,7 +355,7 @@ ILWaitHandle *ILWaitMutexNamedCreate(const char *name, int initiallyOwned,
 	return &(mutex->parent.parent);
 }
 
-int PrivateILWaitMutexRelease(ILWaitHandle *handle, int mode)
+int ILWaitMutexRelease(ILWaitHandle *handle)
 {
 	ILWaitMutex *mutex = (ILWaitMutex *)handle;
 	int result;
@@ -361,55 +367,29 @@ int PrivateILWaitMutexRelease(ILWaitHandle *handle, int mode)
 	if((mutex->parent.kind & IL_WAIT_MUTEX) == 0)
 	{
 		/* This isn't actually a mutex */
-		result = 0;
+		result = IL_WAITMUTEX_RELEASE_FAIL;
 	}
 	else if(mutex->owner != &((ILThreadSelf())->wakeup))
 	{
 		/* This thread doesn't currently own the mutex */
-		result = 0;
-	}
-	else if (mode == 2)
-	{
-		/* Complete a release (after a call with mode = 1) */
-		
-		if (mutex->count == 0)
-		{
-			mutex->owner = _ILWakeupQueueWake(&(mutex->queue));
-			if(mutex->owner != 0)
-			{
-				mutex->count = 1;
-			}
-		}
-
-		result = 1;
+		result = IL_WAITMUTEX_RELEASE_FAIL;
 	}
 	else if(--(mutex->count) == 0)
 	{
 		/* The count has returned to zero, so find something
 		   else to give the ownership of the mutex to */
 
-		if (mode == 0)
+		mutex->owner = _ILWakeupQueueWake(&(mutex->queue));
+		if(mutex->owner != 0)
 		{
-			mutex->owner = _ILWakeupQueueWake(&(mutex->queue));
-			if(mutex->owner != 0)
-			{
-				mutex->count = 1;
-			}
-			result = 1;
-		}
-		else if (mode == 1)
-		{
-			result = 1;
-		}
-		else
-		{
-			result = 0;
+			mutex->count = 1;
 		}
+		result = IL_WAITMUTEX_RELEASE_SUCCESS;
 	}
 	else
 	{
 		/* The current thread still owns the mutex */
-		result = IL_WAIT_LEAVE_STILL_OWNS;
+		result = IL_WAITMUTEX_RELEASE_STILL_OWNS;
 	}
 
 	/* Unlock the mutex and return */
@@ -417,11 +397,6 @@ int PrivateILWaitMutexRelease(ILWaitHandle *handle, int mode)
 	return result;
 }
 
-int ILWaitMutexRelease(ILWaitHandle *handle)
-{
-	return PrivateILWaitMutexRelease(handle, 0);
-}
-
 /*
  * Close a monitor.
  */
@@ -429,8 +404,11 @@ static int MonitorClose(ILMonitor *monitor)
 {
 	/* Lock down the monitor and determine if it is currently owned */
 	_ILMutexLock(&(monitor->parent.parent.lock));
-	if(monitor->parent.owner != 0 ||
-	   !_ILWakeupQueueIsEmpty(&(monitor->parent.queue)))
+	
+	/* We we allow monitors to be closed even if they have
+	   an owner.  It is valid for a program to lock an
+	   object and never release it before it gets GC-ed.  */
+	if(monitor->waiters > 0 || !_ILWakeupQueueIsEmpty(&(monitor->parent.queue)))
 	{
 		_ILMutexUnlock(&(monitor->parent.parent.lock));
 		return IL_WAITCLOSE_OWNED;
@@ -464,6 +442,7 @@ ILWaitHandle *ILWaitMonitorCreate(void)
 	monitor->parent.parent.signalFunc = MutexSignal;
 	monitor->parent.owner = 0;
 	monitor->parent.count = 0;
+	monitor->waiters = 0;
 	_ILWakeupQueueCreate(&(monitor->parent.queue));
 	_ILWakeupQueueCreate(&(monitor->signalQueue));
 
@@ -471,16 +450,42 @@ ILWaitHandle *ILWaitMonitorCreate(void)
 	return &(monitor->parent.parent);
 }
 
+/*
+ * The specification for what happens when a wait is aborted is ambiguous at best.
+ * MS.NET appears to allow interrupted and aborted threads to exit wait without
+ * reaquire the monitor lock.  To facilitate this, MS.NET ignores calls to
+ * Monitor.Exit by any thread that has or ever was aborted.
+ *
+ * See: http://dotgnu.org/pipermail/developers/2004-May/012214.html
+ *      http://dotgnu.org/pipermail/developers/2004-May/012226.html
+ *
+ * In PNET's implementation a waiting thread *must* reaquire the lock on the
+ * monitor it is waiting on before it can continue.  Like, MS.NET, the
+ * interrupted/aborted thread does not need to wait for a pulse.
+ * 
+ * Allowing a waiting thread to exit Monitor.Wait without reaquiring the lock
+ * could potentially lead to deadlocks and data corruption.
+ */
 int ILWaitMonitorWait(ILWaitHandle *handle, ILUInt32 timeout)
 {
+	ILThread *thread = ILThreadSelf();
 	ILMonitor *monitor = (ILMonitor *)handle;
 	_ILWakeup *wakeup = &((ILThreadSelf())->wakeup);
 	int result, result2;
 	unsigned long saveCount;
 
+	result = _ILEnterWait(thread);
+	
+	if (result != 0)
+	{
+		return result;
+	}
+		
 	/* Lock down the monitor */
 	_ILMutexLock(&(monitor->parent.parent.lock));
 
+	++monitor->waiters;
+
 	/* Determine what to do based on the monitor's state */
 	if(monitor->parent.parent.kind != IL_WAIT_MONITOR)
 	{
@@ -499,16 +504,30 @@ int ILWaitMonitorWait(ILWaitHandle *handle, ILUInt32 timeout)
 		monitor->parent.owner = 0;
 		monitor->parent.count = 0;
 
-		/* Add ourselves to the signal wakeup queue */
-		_ILWakeupQueueAdd(&(monitor->signalQueue), wakeup, monitor);
+		/*
+		 * Must set the limit before we add ourselves to the wakeup queue 't
+		 * otherwise we might miss any signal between now and the call to 
+		 * _ILWakeupWait.
+		 */
+		if (_ILWakeupSetLimit(wakeup, 1))
+		{			
+			/* Add ourselves to the signal wakeup queue */
+			_ILWakeupQueueAdd(&(monitor->signalQueue), wakeup, monitor);
 
-		/* Unlock the monitor */
-		_ILMutexUnlock(&(monitor->parent.parent.lock));
+			/* Wakeup any threads waiting to enter */
+			monitor->parent.owner = _ILWakeupQueueWake(&(monitor->parent.queue));
 
-		/* Wait until we are signalled */
-		if(_ILWakeupSetLimit(wakeup, 1))
-		{
+			if(monitor->parent.owner != 0)
+			{			
+				monitor->parent.count = 1;
+			}
+
+			/* Unlock the monitor */
+			_ILMutexUnlock(&(monitor->parent.parent.lock));
+
+			/* Wait until we are signalled */			
 			result = _ILWakeupWait(wakeup, timeout, 0);
+						
 			if(result < 0)
 			{
 				result = IL_WAIT_INTERRUPTED;
@@ -520,18 +539,23 @@ int ILWaitMonitorWait(ILWaitHandle *handle, ILUInt32 timeout)
 		}
 		else
 		{
+			/* Unlock the monitor */
+			_ILMutexUnlock(&(monitor->parent.parent.lock));
+
 			result = IL_WAIT_INTERRUPTED;
 		}
 
-		/* Wait to re-acquire the monitor */
-		result2 = ILWaitOne(handle, IL_WAIT_INFINITE);
-		if(result > 0 && result2 < 0)
+		/* Wait to reaquire the monitor */
+		result2 = _ILWaitOneBackupInterruptsAndAborts(handle, -1);
+
+		if (result2 < 0)
 		{
 			result = result2;
 		}
 
 		/* Lock down the monitor and set the count back to the right value */
 		_ILMutexLock(&(monitor->parent.parent.lock));
+
 		if(monitor->parent.owner == 0)
 		{
 			monitor->parent.owner = wakeup;
@@ -543,12 +567,15 @@ int ILWaitMonitorWait(ILWaitHandle *handle, ILUInt32 timeout)
 		}
 	}
 
+	--monitor->waiters;
+	
 	/* Unlock the monitor and return */
 	_ILMutexUnlock(&(monitor->parent.parent.lock));
-	return result;
+
+	return _ILLeaveWait(thread, result);
 }
 
-int ILWaitMonitorPulse(ILWaitHandle *handle)
+static IL_INLINE int PrivateWaitMonitorPulse(ILWaitHandle *handle, int all)
 {
 	ILMonitor *monitor = (ILMonitor *)handle;
 	_ILWakeup *wakeup = &((ILThreadSelf())->wakeup);
@@ -571,76 +598,32 @@ int ILWaitMonitorPulse(ILWaitHandle *handle)
 	else
 	{
 		/* Wake up something on the signal queue */
-		_ILWakeupQueueWake(&(monitor->signalQueue));
+		/* GCC should optimise out this if statement */
+		
+		if (all)
+		{			
+			_ILWakeupQueueWakeAll(&(monitor->signalQueue));
+		}
+		else
+		{
+			_ILWakeupQueueWake(&(monitor->signalQueue));
+		}
+
 		result = 1;
 	}
-
 	/* Unlock the monitor and return */
 	_ILMutexUnlock(&(monitor->parent.parent.lock));
 	return result;
 }
 
+int ILWaitMonitorPulse(ILWaitHandle *handle)
+{
+	return PrivateWaitMonitorPulse(handle, 0);
+}
+
 int ILWaitMonitorPulseAll(ILWaitHandle *handle)
 {
-	ILMonitor *monitor = (ILMonitor *)handle;
-	_ILWakeup *wakeup = &((ILThreadSelf())->wakeup);
-	int result;
-
-	/* Lock down the monitor */
-	_ILMutexLock(&(monitor->parent.parent.lock));
-
-	/* Determine what to do based on the monitor's state */
-	if(monitor->parent.parent.kind != IL_WAIT_MONITOR)
-	{
-		/* This isn't actually a monitor */
-		result = 0;
-	}
-	else if(monitor->parent.owner != wakeup)
-	{
-		/* This thread doesn't currently own the monitor */
-		result = 0;
-	}
-	else
-	{
-		/* Wake up everything on the signal queue */
-		_ILWakeupQueueWakeAll(&(monitor->signalQueue));
-		result = 1;
-	}
-
-	/* Unlock the monitor and return */
-	_ILMutexUnlock(&(monitor->parent.parent.lock));
-	return result;
-}
-
-int ILWaitMonitorCanClose(ILWaitHandle *handle)
-{
-	ILMonitor *monitor = (ILMonitor *)handle;
-
-	_ILMutexLock(&(monitor->parent.parent.lock));
-
-	if(monitor->parent.count == 0 &&
-		_ILWakeupQueueIsEmpty(&(monitor->parent.queue)))
-	{
-		_ILMutexUnlock(&(monitor->parent.parent.lock));
-
-		return 1;
-	}
-	else
-	{
-		_ILMutexUnlock(&(monitor->parent.parent.lock));
-
-		return 0;	
-	}
-}
-
-int ILWaitMonitorSpeculativeLeave(ILWaitHandle *handle)
-{
-	return PrivateILWaitMutexRelease(handle, 1);
-}
-
-int ILWaitMonitorCompleteLeave(ILWaitHandle *handle)
-{
-	return PrivateILWaitMutexRelease(handle, 2);
+	return PrivateWaitMonitorPulse(handle, 1);
 }
 
 #ifdef	__cplusplus
